# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from cffi import FFI
import re

import pytest

from pywayland.protocol.wayland import (
    WlBuffer,
    WlCallback,
    WlCompositor,
    WlDataDeviceManager,
    WlDataDevice,
    WlDataOffer,
    WlDataSource,
    WlDisplay,
    WlKeyboard,
    WlOutput,
    WlPointer,
    WlRegion,
    WlRegistry,
    WlSeat,
    WlShell,
    WlShellSurface,
    WlShmPool,
    WlShm,
    WlSubcompositor,
    WlSubsurface,
    WlSurface,
    WlTouch,
)

ffi = FFI()

ffi.cdef("""
struct wl_message {
    const char *name;
    const char *signature;
    const struct wl_interface **types;
};
struct wl_interface {
    const char *name;
    int version;
    int method_count;
    const struct wl_message *methods;
    int event_count;
    const struct wl_message *events;
};

extern const struct wl_interface wl_buffer_interface;
extern const struct wl_interface wl_callback_interface;
extern const struct wl_interface wl_compositor_interface;
extern const struct wl_interface wl_data_device_manager_interface;
extern const struct wl_interface wl_data_device_interface;
extern const struct wl_interface wl_data_offer_interface;
extern const struct wl_interface wl_data_source_interface;
extern const struct wl_interface wl_display_interface;
extern const struct wl_interface wl_keyboard_interface;
extern const struct wl_interface wl_output_interface;
extern const struct wl_interface wl_pointer_interface;
extern const struct wl_interface wl_region_interface;
extern const struct wl_interface wl_registry_interface;
extern const struct wl_interface wl_seat_interface;
extern const struct wl_interface wl_shell_interface;
extern const struct wl_interface wl_shell_surface_interface;
extern const struct wl_interface wl_shm_pool_interface;
extern const struct wl_interface wl_shm_interface;
extern const struct wl_interface wl_subcompositor_interface;
extern const struct wl_interface wl_subsurface_interface;
extern const struct wl_interface wl_surface_interface;
extern const struct wl_interface wl_touch_interface;
""")

C = ffi.verify("""
#include <wayland-server-protocol.h>
""", libraries=['wayland-server', 'wayland-client'])

# Check the generated cdata interfaces against actual ones, list of all
# interfaces as of wayland 1.7.0
interfaces = [
    (WlBuffer, C.wl_buffer_interface),
    (WlCallback, C.wl_callback_interface),
    (WlCompositor, C.wl_compositor_interface),
    (WlDataDeviceManager, C.wl_data_device_manager_interface),
    (WlDataDevice, C.wl_data_device_interface),
    (WlDataOffer, C.wl_data_offer_interface),
    (WlDataSource, C.wl_data_source_interface),
    (WlDisplay, C.wl_display_interface),
    (WlKeyboard, C.wl_keyboard_interface),
    (WlOutput, C.wl_output_interface),
    (WlPointer, C.wl_pointer_interface),
    (WlRegion, C.wl_region_interface),
    (WlRegistry, C.wl_registry_interface),
    (WlSeat, C.wl_seat_interface),
    (WlShell, C.wl_shell_interface),
    (WlShellSurface, C.wl_shell_surface_interface),
    (WlShmPool, C.wl_shm_pool_interface),
    (WlShm, C.wl_shm_interface),
    (WlSubcompositor, C.wl_subcompositor_interface),
    (WlSubsurface, C.wl_subsurface_interface),
    (WlSurface, C.wl_surface_interface),
    (WlTouch, C.wl_touch_interface),
]

re_arg = re.compile(r"(\??)([uifsonah])")


def verify_wl_message(py_ptr, wl_ptr):
    """Verify the wl_message

    Check the wl_message associated with the given cdata pointer against the
    given wl_interface object
    """
    assert ffi.string(wl_ptr.name) == ffi.string(py_ptr.name)
    assert ffi.string(wl_ptr.signature) == ffi.string(py_ptr.signature)

    signature = ffi.string(wl_ptr.signature).decode()
    nargs = len(re_arg.findall(signature))
    for j in range(nargs):
        wl_type = wl_ptr.types[j]
        py_type = py_ptr.types[j]
        if wl_type == ffi.NULL:
            assert py_type == ffi.NULL, j
        else:
            assert py_type != ffi.NULL
            assert ffi.string(wl_type.name) == ffi.string(py_type.name)


@pytest.mark.parametrize("py_cls,wl_ptr", interfaces)
def test_wl_interface(py_cls, wl_ptr):
    """ Verify that the wl_interface of the Python class

    Check the wl_interface associated with the given Python class against the
    given wl_interface object
    """
    py_ptr = py_cls._ptr

    assert ffi.string(wl_ptr.name) == ffi.string(py_ptr.name)
    assert wl_ptr.version == py_ptr.version
    assert wl_ptr.event_count == py_ptr.event_count

    assert wl_ptr.method_count == py_ptr.method_count
    for i in range(wl_ptr.method_count):
        verify_wl_message(py_ptr.methods[i], wl_ptr.methods[i])

    assert wl_ptr.event_count == py_ptr.event_count
    for i in range(wl_ptr.event_count):
        verify_wl_message(py_ptr.events[i], wl_ptr.events[i])



# This file has been autogenerated by the pywayland scanner

# Copyright © 2014, 2015 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from .zwp_linux_buffer_params_v1 import ZwpLinuxBufferParamsV1


class ZwpLinuxDmabufV1(Interface):
    """Factory for creating dmabuf-based wl_buffers

    Following the interfaces from:
    https://www.khronos.org/registry/egl/extensions/EXT/EGL_EXT_image_dma_buf_import.txt
    https://www.khronos.org/registry/EGL/extensions/EXT/EGL_EXT_image_dma_buf_import_modifiers.txt
    and the Linux DRM sub-system's AddFb2 ioctl.

    This interface offers ways to create generic dmabuf-based wl_buffers.
    Immediately after a client binds to this interface, the set of supported
    formats and format modifiers is sent with 'format' and 'modifier' events.

    The following are required from clients:

    - Clients must ensure that either all data in the dma-buf is coherent for
      all subsequent read access or that coherency is correctly handled by the
      underlying kernel-side dma-buf implementation.

    - Don't make any more attachments after sending the buffer to the
      compositor. Making more attachments later increases the risk of the
      compositor not being able to use (re-import) an existing dmabuf-based
      :class:`~pywayland.protocol.wayland.WlBuffer`.

    The underlying graphics stack must ensure the following:

    - The dmabuf file descriptors relayed to the server will stay valid for the
      whole lifetime of the :class:`~pywayland.protocol.wayland.WlBuffer`. This
      means the server may at any time use those fds to import the dmabuf into
      any kernel sub-system that might accept it.

    However, when the underlying graphics stack fails to deliver the promise,
    because of e.g. a device hot-unplug which raises internal errors, after the
    :class:`~pywayland.protocol.wayland.WlBuffer` has been successfully created
    the compositor must not raise protocol errors to the client when dmabuf
    import later fails.

    To create a :class:`~pywayland.protocol.wayland.WlBuffer` from one or more
    dmabufs, a client creates a zwp_linux_dmabuf_params_v1 object with a
    :func:`ZwpLinuxDmabufV1.create_params()` request. All planes required by
    the intended format are added with the 'add' request. Finally, a 'create'
    or 'create_immed' request is issued, which has the following outcome
    depending on the import success.

    The 'create' request, - on success, triggers a 'created' event which
    provides the final   :class:`~pywayland.protocol.wayland.WlBuffer` to the
    client. - on failure, triggers a 'failed' event to convey that the server
    cannot use the dmabufs received from the client.

    For the 'create_immed' request, - on success, the server immediately
    imports the added dmabufs to   create a
    :class:`~pywayland.protocol.wayland.WlBuffer`. No event is sent from the
    server in this case. - on failure, the server can choose to either:   -
    terminate the client by raising a fatal error.   - mark the
    :class:`~pywayland.protocol.wayland.WlBuffer` as failed, and send a
    'failed' event to the     client. If the client uses a failed
    :class:`~pywayland.protocol.wayland.WlBuffer` as an argument to any
    request, the behaviour is compositor implementation-defined.

    Warning! The protocol described in this file is experimental and backward
    incompatible changes may be made. Backward compatible changes may be added
    together with the corresponding interface version bump. Backward
    incompatible changes are done by bumping the version number in the protocol
    and interface names and resetting the interface version. Once the protocol
    is to be declared stable, the 'z' prefix and the version number in the
    protocol and interface names are removed and the interface version number
    is reset.
    """

    name = "zwp_linux_dmabuf_v1"
    version = 3


class ZwpLinuxDmabufV1Proxy(Proxy):
    interface = ZwpLinuxDmabufV1

    @ZwpLinuxDmabufV1.request()
    def destroy(self):
        """Unbind the factory

        Objects created through this interface, especially wl_buffers, will
        remain valid.
        """
        self._marshal(0)
        self._destroy()

    @ZwpLinuxDmabufV1.request(
        Argument(ArgumentType.NewId, interface=ZwpLinuxBufferParamsV1),
    )
    def create_params(self):
        """Create a temporary object for buffer parameters

        This temporary object is used to collect multiple dmabuf handles into a
        single batch to create a :class:`~pywayland.protocol.wayland.WlBuffer`.
        It can only be used once and should be destroyed after a 'created' or
        'failed' event has been received.

        :returns:
            :class:`~pywayland.protocol.linux_dmabuf_unstable_v1.ZwpLinuxBufferParamsV1`
            -- the new temporary
        """
        params_id = self._marshal_constructor(1, ZwpLinuxBufferParamsV1)
        return params_id


class ZwpLinuxDmabufV1Resource(Resource):
    interface = ZwpLinuxDmabufV1

    @ZwpLinuxDmabufV1.event(
        Argument(ArgumentType.Uint),
    )
    def format(self, format):
        """Supported buffer format

        This event advertises one buffer format that the server supports. All
        the supported formats are advertised once when the client binds to this
        interface. A roundtrip after binding guarantees that the client has
        received all supported formats.

        For the definition of the format codes, see the
        :class:`~pywayland.protocol.linux_dmabuf_unstable_v1.ZwpLinuxBufferParamsV1`::create
        request.

        Warning: the 'format' event is likely to be deprecated and replaced
        with the 'modifier' event introduced in :class:`ZwpLinuxDmabufV1`
        version 3, described below. Please refrain from using the information
        received from this event.

        :param format:
            DRM_FORMAT code
        :type format:
            `ArgumentType.Uint`
        """
        self._post_event(0, format)

    @ZwpLinuxDmabufV1.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        version=3,
    )
    def modifier(self, format, modifier_hi, modifier_lo):
        """Supported buffer format modifier

        This event advertises the formats that the server supports, along with
        the modifiers supported for each format. All the supported modifiers
        for all the supported formats are advertised once when the client binds
        to this interface. A roundtrip after binding guarantees that the client
        has received all supported format-modifier pairs.

        For legacy support, DRM_FORMAT_MOD_INVALID (that is, modifier_hi ==
        0x00ffffff and modifier_lo == 0xffffffff) is allowed in this event. It
        indicates that the server can support the format with an implicit
        modifier. When a plane has DRM_FORMAT_MOD_INVALID as its modifier, it
        is as if no explicit modifier is specified. The effective modifier will
        be derived from the dmabuf.

        A compositor that sends valid modifiers and DRM_FORMAT_MOD_INVALID for
        a given format supports both explicit modifiers and implicit modifiers.

        For the definition of the format and modifier codes, see the
        :class:`~pywayland.protocol.linux_dmabuf_unstable_v1.ZwpLinuxBufferParamsV1`::create
        and
        :class:`~pywayland.protocol.linux_dmabuf_unstable_v1.ZwpLinuxBufferParamsV1`::add
        requests.

        :param format:
            DRM_FORMAT code
        :type format:
            `ArgumentType.Uint`
        :param modifier_hi:
            high 32 bits of layout modifier
        :type modifier_hi:
            `ArgumentType.Uint`
        :param modifier_lo:
            low 32 bits of layout modifier
        :type modifier_lo:
            `ArgumentType.Uint`
        """
        self._post_event(1, format, modifier_hi, modifier_lo)


class ZwpLinuxDmabufV1Global(Global):
    interface = ZwpLinuxDmabufV1


ZwpLinuxDmabufV1._gen_c()
ZwpLinuxDmabufV1.proxy_class = ZwpLinuxDmabufV1Proxy
ZwpLinuxDmabufV1.resource_class = ZwpLinuxDmabufV1Resource
ZwpLinuxDmabufV1.global_class = ZwpLinuxDmabufV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2011 Kristian Høgsberg
# Copyright © 2010-2011 Intel Corporation
# Copyright © 2012-2013 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource


class WlRegion(Interface):
    """Region interface

    A region object describes an area.

    Region objects are used to describe the opaque and input regions of a
    surface.
    """

    name = "wl_region"
    version = 1


class WlRegionProxy(Proxy):
    interface = WlRegion

    @WlRegion.request()
    def destroy(self):
        """Destroy region

        Destroy the region.  This will invalidate the object ID.
        """
        self._marshal(0)
        self._destroy()

    @WlRegion.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def add(self, x, y, width, height):
        """Add rectangle to region

        Add the specified rectangle to the region.

        :param x:
            region-local x coordinate
        :type x:
            `ArgumentType.Int`
        :param y:
            region-local y coordinate
        :type y:
            `ArgumentType.Int`
        :param width:
            rectangle width
        :type width:
            `ArgumentType.Int`
        :param height:
            rectangle height
        :type height:
            `ArgumentType.Int`
        """
        self._marshal(1, x, y, width, height)

    @WlRegion.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def subtract(self, x, y, width, height):
        """Subtract rectangle from region

        Subtract the specified rectangle from the region.

        :param x:
            region-local x coordinate
        :type x:
            `ArgumentType.Int`
        :param y:
            region-local y coordinate
        :type y:
            `ArgumentType.Int`
        :param width:
            rectangle width
        :type width:
            `ArgumentType.Int`
        :param height:
            rectangle height
        :type height:
            `ArgumentType.Int`
        """
        self._marshal(2, x, y, width, height)


class WlRegionResource(Resource):
    interface = WlRegion


class WlRegionGlobal(Global):
    interface = WlRegion


WlRegion._gen_c()
WlRegion.proxy_class = WlRegionProxy
WlRegion.resource_class = WlRegionResource
WlRegion.global_class = WlRegionGlobal



# This file has been autogenerated by the pywayland scanner

# Copyright © 2012, 2013 Intel Corporation
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from .zwp_input_method_context_v1 import ZwpInputMethodContextV1


class ZwpInputMethodV1(Interface):
    """Input method

    An input method object is responsible for composing text in response to
    input from hardware or virtual keyboards. There is one input method object
    per seat. On activate there is a new input method context object created
    which allows the input method to communicate with the text input.
    """

    name = "zwp_input_method_v1"
    version = 1


class ZwpInputMethodV1Proxy(Proxy):
    interface = ZwpInputMethodV1


class ZwpInputMethodV1Resource(Resource):
    interface = ZwpInputMethodV1

    @ZwpInputMethodV1.event(
        Argument(ArgumentType.NewId, interface=ZwpInputMethodContextV1),
    )
    def activate(self, id):
        """Activate event

        A text input was activated. Creates an input method context object
        which allows communication with the text input.

        :param id:
        :type id:
            :class:`~pywayland.protocol.input_method_unstable_v1.ZwpInputMethodContextV1`
        """
        self._post_event(0, id)

    @ZwpInputMethodV1.event(
        Argument(ArgumentType.Object, interface=ZwpInputMethodContextV1),
    )
    def deactivate(self, context):
        """Deactivate event

        The text input corresponding to the context argument was deactivated.
        The input method context should be destroyed after deactivation is
        handled.

        :param context:
        :type context:
            :class:`~pywayland.protocol.input_method_unstable_v1.ZwpInputMethodContextV1`
        """
        self._post_event(1, context)


class ZwpInputMethodV1Global(Global):
    interface = ZwpInputMethodV1


ZwpInputMethodV1._gen_c()
ZwpInputMethodV1.proxy_class = ZwpInputMethodV1Proxy
ZwpInputMethodV1.resource_class = ZwpInputMethodV1Resource
ZwpInputMethodV1.global_class = ZwpInputMethodV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..wayland import WlSurface


class ZwpPointerGestureSwipeV1(Interface):
    """A swipe gesture object

    A swipe gesture object notifies a client about a multi-finger swipe gesture
    detected on an indirect input device such as a touchpad. The gesture is
    usually initiated by multiple fingers moving in the same direction but once
    initiated the direction may change. The precise conditions of when such a
    gesture is detected are implementation-dependent.

    A gesture consists of three stages: begin, update (optional) and end. There
    cannot be multiple simultaneous hold, pinch or swipe gestures on a same
    pointer/seat, how compositors prevent these situations is implementation-
    dependent.

    A gesture may be cancelled by the compositor or the hardware. Clients
    should not consider performing permanent or irreversible actions until the
    end of a gesture has been received.
    """

    name = "zwp_pointer_gesture_swipe_v1"
    version = 2


class ZwpPointerGestureSwipeV1Proxy(Proxy):
    interface = ZwpPointerGestureSwipeV1

    @ZwpPointerGestureSwipeV1.request()
    def destroy(self):
        """Destroy the pointer swipe gesture object
        """
        self._marshal(0)
        self._destroy()


class ZwpPointerGestureSwipeV1Resource(Resource):
    interface = ZwpPointerGestureSwipeV1

    @ZwpPointerGestureSwipeV1.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Object, interface=WlSurface),
        Argument(ArgumentType.Uint),
    )
    def begin(self, serial, time, surface, fingers):
        """Multi-finger swipe begin

        This event is sent when a multi-finger swipe gesture is detected on the
        device.

        :param serial:
        :type serial:
            `ArgumentType.Uint`
        :param time:
            timestamp with millisecond granularity
        :type time:
            `ArgumentType.Uint`
        :param surface:
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :param fingers:
            number of fingers
        :type fingers:
            `ArgumentType.Uint`
        """
        self._post_event(0, serial, time, surface, fingers)

    @ZwpPointerGestureSwipeV1.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Fixed),
        Argument(ArgumentType.Fixed),
    )
    def update(self, time, dx, dy):
        """Multi-finger swipe motion

        This event is sent when a multi-finger swipe gesture changes the
        position of the logical center.

        The dx and dy coordinates are relative coordinates of the logical
        center of the gesture compared to the previous event.

        :param time:
            timestamp with millisecond granularity
        :type time:
            `ArgumentType.Uint`
        :param dx:
            delta x coordinate in surface coordinate space
        :type dx:
            `ArgumentType.Fixed`
        :param dy:
            delta y coordinate in surface coordinate space
        :type dy:
            `ArgumentType.Fixed`
        """
        self._post_event(1, time, dx, dy)

    @ZwpPointerGestureSwipeV1.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Int),
    )
    def end(self, serial, time, cancelled):
        """Multi-finger swipe end

        This event is sent when a multi-finger swipe gesture ceases to be
        valid. This may happen when one or more fingers are lifted or the
        gesture is cancelled.

        When a gesture is cancelled, the client should undo state changes
        caused by this gesture. What causes a gesture to be cancelled is
        implementation-dependent.

        :param serial:
        :type serial:
            `ArgumentType.Uint`
        :param time:
            timestamp with millisecond granularity
        :type time:
            `ArgumentType.Uint`
        :param cancelled:
            1 if the gesture was cancelled, 0 otherwise
        :type cancelled:
            `ArgumentType.Int`
        """
        self._post_event(2, serial, time, cancelled)


class ZwpPointerGestureSwipeV1Global(Global):
    interface = ZwpPointerGestureSwipeV1


ZwpPointerGestureSwipeV1._gen_c()
ZwpPointerGestureSwipeV1.proxy_class = ZwpPointerGestureSwipeV1Proxy
ZwpPointerGestureSwipeV1.resource_class = ZwpPointerGestureSwipeV1Resource
ZwpPointerGestureSwipeV1.global_class = ZwpPointerGestureSwipeV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2013 Kristian Høgsberg
# Copyright © 2013      Rafael Antognolli
# Copyright © 2013      Jasper St. Pierre
# Copyright © 2010-2013 Intel Corporation
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from .zxdg_popup_v6 import ZxdgPopupV6
from .zxdg_positioner_v6 import ZxdgPositionerV6
from .zxdg_toplevel_v6 import ZxdgToplevelV6


class ZxdgSurfaceV6(Interface):
    """Desktop user interface surface base interface

    An interface that may be implemented by a
    :class:`~pywayland.protocol.wayland.WlSurface`, for implementations that
    provide a desktop-style user interface.

    It provides a base set of functionality required to construct user
    interface elements requiring management by the compositor, such as toplevel
    windows, menus, etc. The types of functionality are split into
    :class:`~pywayland.protocol.xdg_shell.XdgSurface` roles.

    Creating an :class:`~pywayland.protocol.xdg_shell.XdgSurface` does not set
    the role for a :class:`~pywayland.protocol.wayland.WlSurface`. In order to
    map an :class:`~pywayland.protocol.xdg_shell.XdgSurface`, the client must
    create a role-specific object using, e.g., get_toplevel, get_popup. The
    :class:`~pywayland.protocol.wayland.WlSurface` for any given
    :class:`~pywayland.protocol.xdg_shell.XdgSurface` can have at most one
    role, and may not be assigned any role not based on
    :class:`~pywayland.protocol.xdg_shell.XdgSurface`.

    A role must be assigned before any other requests are made to the
    :class:`~pywayland.protocol.xdg_shell.XdgSurface` object.

    The client must call :func:`WlSurface.commit()
    <pywayland.protocol.wayland.WlSurface.commit>` on the corresponding
    :class:`~pywayland.protocol.wayland.WlSurface` for the
    :class:`~pywayland.protocol.xdg_shell.XdgSurface` state to take effect.

    Creating an :class:`~pywayland.protocol.xdg_shell.XdgSurface` from a
    :class:`~pywayland.protocol.wayland.WlSurface` which has a buffer attached
    or committed is a client error, and any attempts by a client to attach or
    manipulate a buffer prior to the first :func:`XdgSurface.configure()
    <pywayland.protocol.xdg_shell.XdgSurface.configure>` call must also be
    treated as errors.

    For a surface to be mapped by the compositor, the following conditions must
    be met: (1) the client has assigned an
    :class:`~pywayland.protocol.xdg_shell.XdgSurface` based role to the
    surface, (2) the client has set and committed the
    :class:`~pywayland.protocol.xdg_shell.XdgSurface` state and the role
    dependent state to the surface and (3) the client has committed a buffer to
    the surface.
    """

    name = "zxdg_surface_v6"
    version = 1

    class error(enum.IntEnum):
        not_constructed = 1
        already_constructed = 2
        unconfigured_buffer = 3


class ZxdgSurfaceV6Proxy(Proxy):
    interface = ZxdgSurfaceV6

    @ZxdgSurfaceV6.request()
    def destroy(self):
        """Destroy the :class:`~pywayland.protocol.xdg_shell.XdgSurface`

        Destroy the :class:`~pywayland.protocol.xdg_shell.XdgSurface` object.
        An :class:`~pywayland.protocol.xdg_shell.XdgSurface` must only be
        destroyed after its role object has been destroyed.
        """
        self._marshal(0)
        self._destroy()

    @ZxdgSurfaceV6.request(
        Argument(ArgumentType.NewId, interface=ZxdgToplevelV6),
    )
    def get_toplevel(self):
        """Assign the :class:`~pywayland.protocol.xdg_shell.XdgToplevel` surface role

        This creates an :class:`~pywayland.protocol.xdg_shell.XdgToplevel`
        object for the given :class:`~pywayland.protocol.xdg_shell.XdgSurface`
        and gives the associated :class:`~pywayland.protocol.wayland.WlSurface`
        the :class:`~pywayland.protocol.xdg_shell.XdgToplevel` role.

        See the documentation of
        :class:`~pywayland.protocol.xdg_shell.XdgToplevel` for more details
        about what an :class:`~pywayland.protocol.xdg_shell.XdgToplevel` is and
        how it is used.

        :returns:
            :class:`~pywayland.protocol.xdg_shell_unstable_v6.ZxdgToplevelV6`
        """
        id = self._marshal_constructor(1, ZxdgToplevelV6)
        return id

    @ZxdgSurfaceV6.request(
        Argument(ArgumentType.NewId, interface=ZxdgPopupV6),
        Argument(ArgumentType.Object, interface=ZxdgSurfaceV6),
        Argument(ArgumentType.Object, interface=ZxdgPositionerV6),
    )
    def get_popup(self, parent, positioner):
        """Assign the :class:`~pywayland.protocol.xdg_shell.XdgPopup` surface role

        This creates an :class:`~pywayland.protocol.xdg_shell.XdgPopup` object
        for the given :class:`~pywayland.protocol.xdg_shell.XdgSurface` and
        gives the associated :class:`~pywayland.protocol.wayland.WlSurface` the
        :class:`~pywayland.protocol.xdg_shell.XdgPopup` role.

        See the documentation of
        :class:`~pywayland.protocol.xdg_shell.XdgPopup` for more details about
        what an :class:`~pywayland.protocol.xdg_shell.XdgPopup` is and how it
        is used.

        :param parent:
        :type parent:
            :class:`ZxdgSurfaceV6`
        :param positioner:
        :type positioner:
            :class:`~pywayland.protocol.xdg_shell_unstable_v6.ZxdgPositionerV6`
        :returns:
            :class:`~pywayland.protocol.xdg_shell_unstable_v6.ZxdgPopupV6`
        """
        id = self._marshal_constructor(2, ZxdgPopupV6, parent, positioner)
        return id

    @ZxdgSurfaceV6.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def set_window_geometry(self, x, y, width, height):
        """Set the new window geometry

        The window geometry of a surface is its "visible bounds" from the
        user's perspective. Client-side decorations often have invisible
        portions like drop-shadows which should be ignored for the purposes of
        aligning, placing and constraining windows.

        The window geometry is double buffered, and will be applied at the time
        :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>` of the corresponding
        :class:`~pywayland.protocol.wayland.WlSurface` is called.

        Once the window geometry of the surface is set, it is not possible to
        unset it, and it will remain the same until set_window_geometry is
        called again, even if a new subsurface or buffer is attached.

        If never set, the value is the full bounds of the surface, including
        any subsurfaces. This updates dynamically on every commit. This unset
        is meant for extremely simple clients.

        The arguments are given in the surface-local coordinate space of the
        :class:`~pywayland.protocol.wayland.WlSurface` associated with this
        :class:`~pywayland.protocol.xdg_shell.XdgSurface`.

        The width and height must be greater than zero. Setting an invalid size
        will raise an error. When applied, the effective window geometry will
        be the set window geometry clamped to the bounding rectangle of the
        combined geometry of the surface of the
        :class:`~pywayland.protocol.xdg_shell.XdgSurface` and the associated
        subsurfaces.

        :param x:
        :type x:
            `ArgumentType.Int`
        :param y:
        :type y:
            `ArgumentType.Int`
        :param width:
        :type width:
            `ArgumentType.Int`
        :param height:
        :type height:
            `ArgumentType.Int`
        """
        self._marshal(3, x, y, width, height)

    @ZxdgSurfaceV6.request(
        Argument(ArgumentType.Uint),
    )
    def ack_configure(self, serial):
        """Ack a configure event

        When a configure event is received, if a client commits the surface in
        response to the configure event, then the client must make an
        ack_configure request sometime before the commit request, passing along
        the serial of the configure event.

        For instance, for toplevel surfaces the compositor might use this
        information to move a surface to the top left only when the client has
        drawn itself for the maximized or fullscreen state.

        If the client receives multiple configure events before it can respond
        to one, it only has to ack the last configure event.

        A client is not required to commit immediately after sending an
        ack_configure request - it may even ack_configure several times before
        its next surface commit.

        A client may send multiple ack_configure requests before committing,
        but only the last request sent before a commit indicates which
        configure event the client really is responding to.

        :param serial:
            the serial from the configure event
        :type serial:
            `ArgumentType.Uint`
        """
        self._marshal(4, serial)


class ZxdgSurfaceV6Resource(Resource):
    interface = ZxdgSurfaceV6

    @ZxdgSurfaceV6.event(
        Argument(ArgumentType.Uint),
    )
    def configure(self, serial):
        """Suggest a surface change

        The configure event marks the end of a configure sequence. A configure
        sequence is a set of one or more events configuring the state of the
        :class:`~pywayland.protocol.xdg_shell.XdgSurface`, including the final
        :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` event.

        Where applicable, :class:`~pywayland.protocol.xdg_shell.XdgSurface`
        surface roles will during a configure sequence extend this event as a
        latched state sent as events before the :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` event. Such events
        should be considered to make up a set of atomically applied
        configuration states, where the :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` commits the
        accumulated state.

        Clients should arrange their surface for the new states, and then send
        an ack_configure request with the serial sent in this configure event
        at some point before committing the new surface.

        If the client receives multiple configure events before it can respond
        to one, it is free to discard all but the last event it received.

        :param serial:
            serial of the configure event
        :type serial:
            `ArgumentType.Uint`
        """
        self._post_event(0, serial)


class ZxdgSurfaceV6Global(Global):
    interface = ZxdgSurfaceV6


ZxdgSurfaceV6._gen_c()
ZxdgSurfaceV6.proxy_class = ZxdgSurfaceV6Proxy
ZxdgSurfaceV6.resource_class = ZxdgSurfaceV6Resource
ZxdgSurfaceV6.global_class = ZxdgSurfaceV6Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2012, 2013 Intel Corporation
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from .zwp_text_input_v1 import ZwpTextInputV1


class ZwpTextInputManagerV1(Interface):
    """Text input manager

    A factory for text_input objects. This object is a global singleton.
    """

    name = "zwp_text_input_manager_v1"
    version = 1


class ZwpTextInputManagerV1Proxy(Proxy):
    interface = ZwpTextInputManagerV1

    @ZwpTextInputManagerV1.request(
        Argument(ArgumentType.NewId, interface=ZwpTextInputV1),
    )
    def create_text_input(self):
        """Create text input

        Creates a new text_input object.

        :returns:
            :class:`~pywayland.protocol.text_input_unstable_v1.ZwpTextInputV1`
        """
        id = self._marshal_constructor(0, ZwpTextInputV1)
        return id


class ZwpTextInputManagerV1Resource(Resource):
    interface = ZwpTextInputManagerV1


class ZwpTextInputManagerV1Global(Global):
    interface = ZwpTextInputManagerV1


ZwpTextInputManagerV1._gen_c()
ZwpTextInputManagerV1.proxy_class = ZwpTextInputManagerV1Proxy
ZwpTextInputManagerV1.resource_class = ZwpTextInputManagerV1Resource
ZwpTextInputManagerV1.global_class = ZwpTextInputManagerV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2013-2014 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..wayland import WlSurface
from .wp_presentation_feedback import WpPresentationFeedback


class WpPresentation(Interface):
    """Timed presentation related :class:`~pywayland.protocol.wayland.WlSurface` requests

    The main feature of this interface is accurate presentation timing feedback
    to ensure smooth video playback while maintaining audio/video
    synchronization. Some features use the concept of a presentation clock,
    which is defined in the presentation.clock_id event.

    A content update for a :class:`~pywayland.protocol.wayland.WlSurface` is
    submitted by a :func:`WlSurface.commit()
    <pywayland.protocol.wayland.WlSurface.commit>` request. Request 'feedback'
    associates with the :func:`WlSurface.commit()
    <pywayland.protocol.wayland.WlSurface.commit>` and provides feedback on the
    content update, particularly the final realized presentation time.



    When the final realized presentation time is available, e.g. after a
    framebuffer flip completes, the requested presentation_feedback.presented
    events are sent. The final presentation time can differ from the
    compositor's predicted display update time and the update's target time,
    especially when the compositor misses its target vertical blanking period.
    """

    name = "wp_presentation"
    version = 1

    class error(enum.IntEnum):
        invalid_timestamp = 0
        invalid_flag = 1


class WpPresentationProxy(Proxy):
    interface = WpPresentation

    @WpPresentation.request()
    def destroy(self):
        """Unbind from the presentation interface

        Informs the server that the client will no longer be using this
        protocol object. Existing objects created by this object are not
        affected.
        """
        self._marshal(0)
        self._destroy()

    @WpPresentation.request(
        Argument(ArgumentType.Object, interface=WlSurface),
        Argument(ArgumentType.NewId, interface=WpPresentationFeedback),
    )
    def feedback(self, surface):
        """Request presentation feedback information

        Request presentation feedback for the current content submission on the
        given surface. This creates a new presentation_feedback object, which
        will deliver the feedback information once. If multiple
        presentation_feedback objects are created for the same submission, they
        will all deliver the same information.

        For details on what information is returned, see the
        presentation_feedback interface.

        :param surface:
            target surface
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :returns:
            :class:`~pywayland.protocol.presentation_time.WpPresentationFeedback`
            -- new feedback object
        """
        callback = self._marshal_constructor(1, WpPresentationFeedback, surface)
        return callback


class WpPresentationResource(Resource):
    interface = WpPresentation

    @WpPresentation.event(
        Argument(ArgumentType.Uint),
    )
    def clock_id(self, clk_id):
        """Clock id for timestamps

        This event tells the client in which clock domain the compositor
        interprets the timestamps used by the presentation extension. This
        clock is called the presentation clock.

        The compositor sends this event when the client binds to the
        presentation interface. The presentation clock does not change during
        the lifetime of the client connection.

        The clock identifier is platform dependent. On Linux/glibc, the
        identifier value is one of the clockid_t values accepted by
        clock_gettime(). clock_gettime() is defined by POSIX.1-2001.

        Timestamps in this clock domain are expressed as tv_sec_hi, tv_sec_lo,
        tv_nsec triples, each component being an unsigned 32-bit value. Whole
        seconds are in tv_sec which is a 64-bit value combined from tv_sec_hi
        and tv_sec_lo, and the additional fractional part in tv_nsec as
        nanoseconds. Hence, for valid timestamps tv_nsec must be in [0,
        999999999].

        Note that clock_id applies only to the presentation clock, and implies
        nothing about e.g. the timestamps used in the Wayland core protocol
        input events.

        Compositors should prefer a clock which does not jump and is not slewed
        e.g. by NTP. The absolute value of the clock is irrelevant. Precision
        of one millisecond or better is recommended. Clients must be able to
        query the current clock value directly, not by asking the compositor.

        :param clk_id:
            platform clock identifier
        :type clk_id:
            `ArgumentType.Uint`
        """
        self._post_event(0, clk_id)


class WpPresentationGlobal(Global):
    interface = WpPresentation


WpPresentation._gen_c()
WpPresentation.proxy_class = WpPresentationProxy
WpPresentation.resource_class = WpPresentationResource
WpPresentation.global_class = WpPresentationGlobal



# This file has been autogenerated by the pywayland scanner

# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pywayland.protocol_core import Global, Interface, Proxy, Resource


class WlXfail(Interface):
    """Xfailing interface

    Items that do not really work yet are put in here, they should be moved
    once they shart working.
    """

    name = "wl_xfail"
    version = 1


class WlXfailProxy(Proxy):
    interface = WlXfail


class WlXfailResource(Resource):
    interface = WlXfail


class WlXfailGlobal(Global):
    interface = WlXfail


WlXfail._gen_c()
WlXfail.proxy_class = WlXfailProxy
WlXfail.resource_class = WlXfailResource
WlXfail.global_class = WlXfailGlobal



# This file has been autogenerated by the pywayland scanner

# Copyright 2014 © Stephen "Lyude" Chandler Paul
# Copyright 2015-2016 © Red Hat, Inc.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from .zwp_tablet_manager_v2 import ZwpTabletManagerV2  # noqa: F401
from .zwp_tablet_pad_group_v2 import ZwpTabletPadGroupV2  # noqa: F401
from .zwp_tablet_pad_ring_v2 import ZwpTabletPadRingV2  # noqa: F401
from .zwp_tablet_pad_strip_v2 import ZwpTabletPadStripV2  # noqa: F401
from .zwp_tablet_pad_v2 import ZwpTabletPadV2  # noqa: F401
from .zwp_tablet_seat_v2 import ZwpTabletSeatV2  # noqa: F401
from .zwp_tablet_tool_v2 import ZwpTabletToolV2  # noqa: F401
from .zwp_tablet_v2 import ZwpTabletV2  # noqa: F401



# This file has been autogenerated by the pywayland scanner

# Copyright 2014 © Stephen "Lyude" Chandler Paul
# Copyright 2015-2016 © Red Hat, Inc.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..wayland import WlSeat
from .zwp_tablet_seat_v1 import ZwpTabletSeatV1


class ZwpTabletManagerV1(Interface):
    """Controller object for graphic tablet devices

    An object that provides access to the graphics tablets available on this
    system. All tablets are associated with a seat, to get access to the actual
    tablets, use wp_tablet_manager.get_tablet_seat.
    """

    name = "zwp_tablet_manager_v1"
    version = 1


class ZwpTabletManagerV1Proxy(Proxy):
    interface = ZwpTabletManagerV1

    @ZwpTabletManagerV1.request(
        Argument(ArgumentType.NewId, interface=ZwpTabletSeatV1),
        Argument(ArgumentType.Object, interface=WlSeat),
    )
    def get_tablet_seat(self, seat):
        """Get the tablet seat

        Get the wp_tablet_seat object for the given seat. This object provides
        access to all graphics tablets in this seat.

        :param seat:
            The :class:`~pywayland.protocol.wayland.WlSeat` object to retrieve
            the tablets for
        :type seat:
            :class:`~pywayland.protocol.wayland.WlSeat`
        :returns:
            :class:`~pywayland.protocol.tablet_unstable_v1.ZwpTabletSeatV1`
        """
        tablet_seat = self._marshal_constructor(0, ZwpTabletSeatV1, seat)
        return tablet_seat

    @ZwpTabletManagerV1.request()
    def destroy(self):
        """Release the memory for the tablet manager object

        Destroy the wp_tablet_manager object. Objects created from this object
        are unaffected and should be destroyed separately.
        """
        self._marshal(1)
        self._destroy()


class ZwpTabletManagerV1Resource(Resource):
    interface = ZwpTabletManagerV1


class ZwpTabletManagerV1Global(Global):
    interface = ZwpTabletManagerV1


ZwpTabletManagerV1._gen_c()
ZwpTabletManagerV1.proxy_class = ZwpTabletManagerV1Proxy
ZwpTabletManagerV1.resource_class = ZwpTabletManagerV1Resource
ZwpTabletManagerV1.global_class = ZwpTabletManagerV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2011 Kristian Høgsberg
# Copyright © 2010-2011 Intel Corporation
# Copyright © 2012-2013 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from .wl_buffer import WlBuffer
from .wl_callback import WlCallback
from .wl_output import WlOutput
from .wl_region import WlRegion


class WlSurface(Interface):
    """An onscreen surface

    A surface is a rectangular area that may be displayed on zero or more
    outputs, and shown any number of times at the compositor's discretion. They
    can present wl_buffers, receive user input, and define a local coordinate
    system.

    The size of a surface (and relative positions on it) is described in
    surface-local coordinates, which may differ from the buffer coordinates of
    the pixel content, in case a buffer_transform or a buffer_scale is used.

    A surface without a "role" is fairly useless: a compositor does not know
    where, when or how to present it. The role is the purpose of a
    :class:`WlSurface`. Examples of roles are a cursor for a pointer (as set by
    :func:`WlPointer.set_cursor()
    <pywayland.protocol.wayland.WlPointer.set_cursor>`), a drag icon
    (:func:`WlDataDevice.start_drag()
    <pywayland.protocol.wayland.WlDataDevice.start_drag>`), a sub-surface
    (:func:`WlSubcompositor.get_subsurface()
    <pywayland.protocol.wayland.WlSubcompositor.get_subsurface>`), and a window
    as defined by a shell protocol (e.g. :func:`WlShell.get_shell_surface()
    <pywayland.protocol.wayland.WlShell.get_shell_surface>`).

    A surface can have only one role at a time. Initially a :class:`WlSurface`
    does not have a role. Once a :class:`WlSurface` is given a role, it is set
    permanently for the whole lifetime of the :class:`WlSurface` object. Giving
    the current role again is allowed, unless explicitly forbidden by the
    relevant interface specification.

    Surface roles are given by requests in other interfaces such as
    :func:`WlPointer.set_cursor()
    <pywayland.protocol.wayland.WlPointer.set_cursor>`. The request should
    explicitly mention that this request gives a role to a :class:`WlSurface`.
    Often, this request also creates a new protocol object that represents the
    role and adds additional functionality to :class:`WlSurface`. When a client
    wants to destroy a :class:`WlSurface`, they must destroy this 'role object'
    before the :class:`WlSurface`.

    Destroying the role object does not remove the role from the
    :class:`WlSurface`, but it may stop the :class:`WlSurface` from "playing
    the role". For instance, if a
    :class:`~pywayland.protocol.wayland.WlSubsurface` object is destroyed, the
    :class:`WlSurface` it was created for will be unmapped and forget its
    position and z-order. It is allowed to create a
    :class:`~pywayland.protocol.wayland.WlSubsurface` for the same
    :class:`WlSurface` again, but it is not allowed to use the
    :class:`WlSurface` as a cursor (cursor is a different role than sub-
    surface, and role switching is not allowed).
    """

    name = "wl_surface"
    version = 4

    class error(enum.IntEnum):
        invalid_scale = 0
        invalid_transform = 1
        invalid_size = 2


class WlSurfaceProxy(Proxy):
    interface = WlSurface

    @WlSurface.request()
    def destroy(self):
        """Delete surface

        Deletes the surface and invalidates its object ID.
        """
        self._marshal(0)
        self._destroy()

    @WlSurface.request(
        Argument(ArgumentType.Object, interface=WlBuffer, nullable=True),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def attach(self, buffer, x, y):
        """Set the surface contents

        Set a buffer as the content of this surface.

        The new size of the surface is calculated based on the buffer size
        transformed by the inverse buffer_transform and the inverse
        buffer_scale. This means that at commit time the supplied buffer size
        must be an integer multiple of the buffer_scale. If that's not the
        case, an invalid_size error is sent.

        The x and y arguments specify the location of the new pending buffer's
        upper left corner, relative to the current buffer's upper left corner,
        in surface-local coordinates. In other words, the x and y, combined
        with the new surface size define in which directions the surface's size
        changes.

        Surface contents are double-buffered state, see
        :func:`WlSurface.commit()`.

        The initial surface contents are void; there is no content.
        :func:`WlSurface.attach()` assigns the given
        :class:`~pywayland.protocol.wayland.WlBuffer` as the pending
        :class:`~pywayland.protocol.wayland.WlBuffer`.
        :func:`WlSurface.commit()` makes the pending
        :class:`~pywayland.protocol.wayland.WlBuffer` the new surface contents,
        and the size of the surface becomes the size calculated from the
        :class:`~pywayland.protocol.wayland.WlBuffer`, as described above.
        After commit, there is no pending buffer until the next attach.

        Committing a pending :class:`~pywayland.protocol.wayland.WlBuffer`
        allows the compositor to read the pixels in the
        :class:`~pywayland.protocol.wayland.WlBuffer`. The compositor may
        access the pixels at any time after the :func:`WlSurface.commit()`
        request. When the compositor will not access the pixels anymore, it
        will send the :func:`WlBuffer.release()
        <pywayland.protocol.wayland.WlBuffer.release>` event. Only after
        receiving :func:`WlBuffer.release()
        <pywayland.protocol.wayland.WlBuffer.release>`, the client may reuse
        the :class:`~pywayland.protocol.wayland.WlBuffer`. A
        :class:`~pywayland.protocol.wayland.WlBuffer` that has been attached
        and then replaced by another attach instead of committed will not
        receive a release event, and is not used by the compositor.

        If a pending :class:`~pywayland.protocol.wayland.WlBuffer` has been
        committed to more than one :class:`WlSurface`, the delivery of
        :func:`WlBuffer.release()
        <pywayland.protocol.wayland.WlBuffer.release>` events becomes
        undefined. A well behaved client should not rely on
        :func:`WlBuffer.release()
        <pywayland.protocol.wayland.WlBuffer.release>` events in this case.
        Alternatively, a client could create multiple
        :class:`~pywayland.protocol.wayland.WlBuffer` objects from the same
        backing storage or use wp_linux_buffer_release.

        Destroying the :class:`~pywayland.protocol.wayland.WlBuffer` after
        :func:`WlBuffer.release()
        <pywayland.protocol.wayland.WlBuffer.release>` does not change the
        surface contents. However, if the client destroys the
        :class:`~pywayland.protocol.wayland.WlBuffer` before receiving the
        :func:`WlBuffer.release()
        <pywayland.protocol.wayland.WlBuffer.release>` event, the surface
        contents become undefined immediately.

        If :func:`WlSurface.attach()` is sent with a NULL
        :class:`~pywayland.protocol.wayland.WlBuffer`, the following
        :func:`WlSurface.commit()` will remove the surface content.

        :param buffer:
            buffer of surface contents
        :type buffer:
            :class:`~pywayland.protocol.wayland.WlBuffer` or `None`
        :param x:
            surface-local x coordinate
        :type x:
            `ArgumentType.Int`
        :param y:
            surface-local y coordinate
        :type y:
            `ArgumentType.Int`
        """
        self._marshal(1, buffer, x, y)

    @WlSurface.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def damage(self, x, y, width, height):
        """Mark part of the surface damaged

        This request is used to describe the regions where the pending buffer
        is different from the current surface contents, and where the surface
        therefore needs to be repainted. The compositor ignores the parts of
        the damage that fall outside of the surface.

        Damage is double-buffered state, see :func:`WlSurface.commit()`.

        The damage rectangle is specified in surface-local coordinates, where x
        and y specify the upper left corner of the damage rectangle.

        The initial value for pending damage is empty: no damage.
        :func:`WlSurface.damage()` adds pending damage: the new pending damage
        is the union of old pending damage and the given rectangle.

        :func:`WlSurface.commit()` assigns pending damage as the current
        damage, and clears pending damage. The server will clear the current
        damage as it repaints the surface.

        Note! New clients should not use this request. Instead damage can be
        posted with :func:`WlSurface.damage_buffer()` which uses buffer
        coordinates instead of surface coordinates.

        :param x:
            surface-local x coordinate
        :type x:
            `ArgumentType.Int`
        :param y:
            surface-local y coordinate
        :type y:
            `ArgumentType.Int`
        :param width:
            width of damage rectangle
        :type width:
            `ArgumentType.Int`
        :param height:
            height of damage rectangle
        :type height:
            `ArgumentType.Int`
        """
        self._marshal(2, x, y, width, height)

    @WlSurface.request(
        Argument(ArgumentType.NewId, interface=WlCallback),
    )
    def frame(self):
        """Request a frame throttling hint

        Request a notification when it is a good time to start drawing a new
        frame, by creating a frame callback. This is useful for throttling
        redrawing operations, and driving animations.

        When a client is animating on a :class:`WlSurface`, it can use the
        'frame' request to get notified when it is a good time to draw and
        commit the next frame of animation. If the client commits an update
        earlier than that, it is likely that some updates will not make it to
        the display, and the client is wasting resources by drawing too often.

        The frame request will take effect on the next
        :func:`WlSurface.commit()`. The notification will only be posted for
        one frame unless requested again. For a :class:`WlSurface`, the
        notifications are posted in the order the frame requests were
        committed.

        The server must send the notifications so that a client will not send
        excessive updates, while still allowing the highest possible update
        rate for clients that wait for the reply before drawing again. The
        server should give some time for the client to draw and commit after
        sending the frame callback events to let it hit the next output
        refresh.

        A server should avoid signaling the frame callbacks if the surface is
        not visible in any way, e.g. the surface is off-screen, or completely
        obscured by other opaque surfaces.

        The object returned by this request will be destroyed by the compositor
        after the callback is fired and as such the client must not attempt to
        use it after that point.

        The callback_data passed in the callback is the current time, in
        milliseconds, with an undefined base.

        :returns:
            :class:`~pywayland.protocol.wayland.WlCallback` -- callback object
            for the frame request
        """
        callback = self._marshal_constructor(3, WlCallback)
        return callback

    @WlSurface.request(
        Argument(ArgumentType.Object, interface=WlRegion, nullable=True),
    )
    def set_opaque_region(self, region):
        """Set opaque region

        This request sets the region of the surface that contains opaque
        content.

        The opaque region is an optimization hint for the compositor that lets
        it optimize the redrawing of content behind opaque regions.  Setting an
        opaque region is not required for correct behaviour, but marking
        transparent content as opaque will result in repaint artifacts.

        The opaque region is specified in surface-local coordinates.

        The compositor ignores the parts of the opaque region that fall outside
        of the surface.

        Opaque region is double-buffered state, see :func:`WlSurface.commit()`.

        :func:`WlSurface.set_opaque_region()` changes the pending opaque
        region. :func:`WlSurface.commit()` copies the pending region to the
        current region. Otherwise, the pending and current regions are never
        changed.

        The initial value for an opaque region is empty. Setting the pending
        opaque region has copy semantics, and the
        :class:`~pywayland.protocol.wayland.WlRegion` object can be destroyed
        immediately. A NULL :class:`~pywayland.protocol.wayland.WlRegion`
        causes the pending opaque region to be set to empty.

        :param region:
            opaque region of the surface
        :type region:
            :class:`~pywayland.protocol.wayland.WlRegion` or `None`
        """
        self._marshal(4, region)

    @WlSurface.request(
        Argument(ArgumentType.Object, interface=WlRegion, nullable=True),
    )
    def set_input_region(self, region):
        """Set input region

        This request sets the region of the surface that can receive pointer
        and touch events.

        Input events happening outside of this region will try the next surface
        in the server surface stack. The compositor ignores the parts of the
        input region that fall outside of the surface.

        The input region is specified in surface-local coordinates.

        Input region is double-buffered state, see :func:`WlSurface.commit()`.

        :func:`WlSurface.set_input_region()` changes the pending input region.
        :func:`WlSurface.commit()` copies the pending region to the current
        region. Otherwise the pending and current regions are never changed,
        except cursor and icon surfaces are special cases, see
        :func:`WlPointer.set_cursor()
        <pywayland.protocol.wayland.WlPointer.set_cursor>` and
        :func:`WlDataDevice.start_drag()
        <pywayland.protocol.wayland.WlDataDevice.start_drag>`.

        The initial value for an input region is infinite. That means the whole
        surface will accept input. Setting the pending input region has copy
        semantics, and the :class:`~pywayland.protocol.wayland.WlRegion` object
        can be destroyed immediately. A NULL
        :class:`~pywayland.protocol.wayland.WlRegion` causes the input region
        to be set to infinite.

        :param region:
            input region of the surface
        :type region:
            :class:`~pywayland.protocol.wayland.WlRegion` or `None`
        """
        self._marshal(5, region)

    @WlSurface.request()
    def commit(self):
        """Commit pending surface state

        Surface state (input, opaque, and damage regions, attached buffers,
        etc.) is double-buffered. Protocol requests modify the pending state,
        as opposed to the current state in use by the compositor. A commit
        request atomically applies all pending state, replacing the current
        state. After commit, the new pending state is as documented for each
        related request.

        On commit, a pending :class:`~pywayland.protocol.wayland.WlBuffer` is
        applied first, and all other state second. This means that all
        coordinates in double-buffered state are relative to the new
        :class:`~pywayland.protocol.wayland.WlBuffer` coming into use, except
        for :func:`WlSurface.attach()` itself. If there is no pending
        :class:`~pywayland.protocol.wayland.WlBuffer`, the coordinates are
        relative to the current surface contents.

        All requests that need a commit to become effective are documented to
        affect double-buffered state.

        Other interfaces may add further double-buffered surface state.
        """
        self._marshal(6)

    @WlSurface.request(
        Argument(ArgumentType.Int),
        version=2,
    )
    def set_buffer_transform(self, transform):
        """Sets the buffer transformation

        This request sets an optional transformation on how the compositor
        interprets the contents of the buffer attached to the surface. The
        accepted values for the transform parameter are the values for
        :func:`WlOutput.transform()
        <pywayland.protocol.wayland.WlOutput.transform>`.

        Buffer transform is double-buffered state, see
        :func:`WlSurface.commit()`.

        A newly created surface has its buffer transformation set to normal.

        :func:`WlSurface.set_buffer_transform()` changes the pending buffer
        transformation. :func:`WlSurface.commit()` copies the pending buffer
        transformation to the current one. Otherwise, the pending and current
        values are never changed.

        The purpose of this request is to allow clients to render content
        according to the output transform, thus permitting the compositor to
        use certain optimizations even if the display is rotated. Using
        hardware overlays and scanning out a client buffer for fullscreen
        surfaces are examples of such optimizations. Those optimizations are
        highly dependent on the compositor implementation, so the use of this
        request should be considered on a case-by-case basis.

        Note that if the transform value includes 90 or 270 degree rotation,
        the width of the buffer will become the surface height and the height
        of the buffer will become the surface width.

        If transform is not one of the values from the
        :func:`WlOutput.transform()
        <pywayland.protocol.wayland.WlOutput.transform>` enum the
        invalid_transform protocol error is raised.

        :param transform:
            transform for interpreting buffer contents
        :type transform:
            `ArgumentType.Int`
        """
        self._marshal(7, transform)

    @WlSurface.request(
        Argument(ArgumentType.Int),
        version=3,
    )
    def set_buffer_scale(self, scale):
        """Sets the buffer scaling factor

        This request sets an optional scaling factor on how the compositor
        interprets the contents of the buffer attached to the window.

        Buffer scale is double-buffered state, see :func:`WlSurface.commit()`.

        A newly created surface has its buffer scale set to 1.

        :func:`WlSurface.set_buffer_scale()` changes the pending buffer scale.
        :func:`WlSurface.commit()` copies the pending buffer scale to the
        current one. Otherwise, the pending and current values are never
        changed.

        The purpose of this request is to allow clients to supply higher
        resolution buffer data for use on high resolution outputs. It is
        intended that you pick the same buffer scale as the scale of the output
        that the surface is displayed on. This means the compositor can avoid
        scaling when rendering the surface on that output.

        Note that if the scale is larger than 1, then you have to attach a
        buffer that is larger (by a factor of scale in each dimension) than the
        desired surface size.

        If scale is not positive the invalid_scale protocol error is raised.

        :param scale:
            positive scale for interpreting buffer contents
        :type scale:
            `ArgumentType.Int`
        """
        self._marshal(8, scale)

    @WlSurface.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        version=4,
    )
    def damage_buffer(self, x, y, width, height):
        """Mark part of the surface damaged using buffer coordinates

        This request is used to describe the regions where the pending buffer
        is different from the current surface contents, and where the surface
        therefore needs to be repainted. The compositor ignores the parts of
        the damage that fall outside of the surface.

        Damage is double-buffered state, see :func:`WlSurface.commit()`.

        The damage rectangle is specified in buffer coordinates, where x and y
        specify the upper left corner of the damage rectangle.

        The initial value for pending damage is empty: no damage.
        :func:`WlSurface.damage_buffer()` adds pending damage: the new pending
        damage is the union of old pending damage and the given rectangle.

        :func:`WlSurface.commit()` assigns pending damage as the current
        damage, and clears pending damage. The server will clear the current
        damage as it repaints the surface.

        This request differs from :func:`WlSurface.damage()` in only one way -
        it takes damage in buffer coordinates instead of surface-local
        coordinates. While this generally is more intuitive than surface
        coordinates, it is especially desirable when using
        :class:`~pywayland.protocol.viewporter.WpViewport` or when a drawing
        library (like EGL) is unaware of buffer scale and buffer transform.

        Note: Because buffer transformation changes and damage requests may be
        interleaved in the protocol stream, it is impossible to determine the
        actual mapping between surface and buffer damage until
        :func:`WlSurface.commit()` time. Therefore, compositors wishing to take
        both kinds of damage into account will have to accumulate damage from
        the two requests separately and only transform from one to the other
        after receiving the :func:`WlSurface.commit()`.

        :param x:
            buffer-local x coordinate
        :type x:
            `ArgumentType.Int`
        :param y:
            buffer-local y coordinate
        :type y:
            `ArgumentType.Int`
        :param width:
            width of damage rectangle
        :type width:
            `ArgumentType.Int`
        :param height:
            height of damage rectangle
        :type height:
            `ArgumentType.Int`
        """
        self._marshal(9, x, y, width, height)


class WlSurfaceResource(Resource):
    interface = WlSurface

    @WlSurface.event(
        Argument(ArgumentType.Object, interface=WlOutput),
    )
    def enter(self, output):
        """Surface enters an output

        This is emitted whenever a surface's creation, movement, or resizing
        results in some part of it being within the scanout region of an
        output.

        Note that a surface may be overlapping with zero or more outputs.

        :param output:
            output entered by the surface
        :type output:
            :class:`~pywayland.protocol.wayland.WlOutput`
        """
        self._post_event(0, output)

    @WlSurface.event(
        Argument(ArgumentType.Object, interface=WlOutput),
    )
    def leave(self, output):
        """Surface leaves an output

        This is emitted whenever a surface's creation, movement, or resizing
        results in it no longer having any part of it within the scanout region
        of an output.

        Clients should not use the number of outputs the surface is on for
        frame throttling purposes. The surface might be hidden even if no leave
        event has been sent, and the compositor might expect new surface
        content updates even if no enter event has been sent. The frame event
        should be used instead.

        :param output:
            output left by the surface
        :type output:
            :class:`~pywayland.protocol.wayland.WlOutput`
        """
        self._post_event(1, output)


class WlSurfaceGlobal(Global):
    interface = WlSurface


WlSurface._gen_c()
WlSurface.proxy_class = WlSurfaceProxy
WlSurface.resource_class = WlSurfaceResource
WlSurface.global_class = WlSurfaceGlobal



# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2011 Kristian Høgsberg
# Copyright © 2010-2011 Intel Corporation
# Copyright © 2012-2013 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource


class WlCallback(Interface):
    """Callback object

    Clients can handle the 'done' event to get notified when the related
    request is done.
    """

    name = "wl_callback"
    version = 1


class WlCallbackProxy(Proxy):
    interface = WlCallback


class WlCallbackResource(Resource):
    interface = WlCallback

    @WlCallback.event(
        Argument(ArgumentType.Uint),
    )
    def done(self, callback_data):
        """Done event

        Notify the client when the related request is done.

        :param callback_data:
            request-specific data for the callback
        :type callback_data:
            `ArgumentType.Uint`
        """
        self._post_event(0, callback_data)


class WlCallbackGlobal(Global):
    interface = WlCallback


WlCallback._gen_c()
WlCallback.proxy_class = WlCallbackProxy
WlCallback.resource_class = WlCallbackResource
WlCallback.global_class = WlCallbackGlobal



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import List, Optional, TypeVar, Type
import abc
import textwrap
import xml.etree.ElementTree as ET

T = TypeVar("T", bound="Element")


class Element(abc.ABC):
    @classmethod
    @abc.abstractmethod
    def parse(cls: Type[T], element: ET.Element) -> T:
        pass

    @staticmethod
    def parse_attribute(element: ET.Element, name: str) -> str:
        obj = Element.parse_optional_attribute(element, name)
        if obj is None:
            raise ValueError()

        return obj

    @staticmethod
    def parse_optional_attribute(element: ET.Element, name) -> Optional[str]:
        obj = element.attrib.get(name)
        return obj

    @staticmethod
    def parse_child(element: ET.Element, child_class: Type[T], name: str) -> T:
        obj = Element.parse_optional_child(element, child_class, name)
        if obj is None:
            raise ValueError()

        return obj

    @staticmethod
    def parse_optional_child(
        element: ET.Element, child_class: Type[T], name: str
    ) -> Optional[T]:
        obj = element.find(name)
        if obj is None:
            return None

        return child_class.parse(obj)

    @staticmethod
    def parse_repeated_child(
        element: ET.Element, child_class: Type[T], name: str
    ) -> List[T]:
        obj = [child_class.parse(elem) for elem in element.findall(name)]
        return obj

    @staticmethod
    def parse_pcdata(element: ET.Element) -> Optional[str]:
        text = element.text
        if text:
            # We need to strip each line while keeping paragraph breaks
            return textwrap.dedent(text.expandtabs(8).rstrip().lstrip("\n"))

        return None



# This file has been autogenerated by the pywayland scanner

# Copyright © 2016 Yong Bakos
# Copyright © 2015 Jason Ekstrand
# Copyright © 2015 Jonas Ådahl
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.protocol_core import Global, Interface, Proxy, Resource


class ZwpFullscreenShellModeFeedbackV1(Interface):
    name = "zwp_fullscreen_shell_mode_feedback_v1"
    version = 1


class ZwpFullscreenShellModeFeedbackV1Proxy(Proxy):
    interface = ZwpFullscreenShellModeFeedbackV1


class ZwpFullscreenShellModeFeedbackV1Resource(Resource):
    interface = ZwpFullscreenShellModeFeedbackV1

    @ZwpFullscreenShellModeFeedbackV1.event()
    def mode_successful(self):
        """Mode switch succeeded

        This event indicates that the attempted mode switch operation was
        successful.  A surface of the size requested in the mode switch will
        fill the output without scaling.

        Upon receiving this event, the client should destroy the
        wl_fullscreen_shell_mode_feedback object.
        """
        self._post_event(0)

    @ZwpFullscreenShellModeFeedbackV1.event()
    def mode_failed(self):
        """Mode switch failed

        This event indicates that the attempted mode switch operation failed.
        This may be because the requested output mode is not possible or it may
        mean that the compositor does not want to allow it.

        Upon receiving this event, the client should destroy the
        wl_fullscreen_shell_mode_feedback object.
        """
        self._post_event(1)

    @ZwpFullscreenShellModeFeedbackV1.event()
    def present_cancelled(self):
        """Mode switch cancelled

        This event indicates that the attempted mode switch operation was
        cancelled.  Most likely this is because the client requested a second
        mode switch before the first one completed.

        Upon receiving this event, the client should destroy the
        wl_fullscreen_shell_mode_feedback object.
        """
        self._post_event(2)


class ZwpFullscreenShellModeFeedbackV1Global(Global):
    interface = ZwpFullscreenShellModeFeedbackV1


ZwpFullscreenShellModeFeedbackV1._gen_c()
ZwpFullscreenShellModeFeedbackV1.proxy_class = ZwpFullscreenShellModeFeedbackV1Proxy
ZwpFullscreenShellModeFeedbackV1.resource_class = ZwpFullscreenShellModeFeedbackV1Resource
ZwpFullscreenShellModeFeedbackV1.global_class = ZwpFullscreenShellModeFeedbackV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2018 NXP
# Copyright © 2019 Status Research & Development GmbH.
# Copyright © 2021 Xaver Hugl
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from .wp_drm_lease_connector_v1 import WpDrmLeaseConnectorV1
from .wp_drm_lease_v1 import WpDrmLeaseV1


class WpDrmLeaseRequestV1(Interface):
    """Drm lease request

    A client that wishes to lease DRM resources will attach the list of
    connectors advertised with :func:`WpDrmLeaseDeviceV1.connector()
    <pywayland.protocol.drm_lease_v1.WpDrmLeaseDeviceV1.connector>` that they
    wish to lease, then use :func:`WpDrmLeaseRequestV1.submit()` to submit the
    request.
    """

    name = "wp_drm_lease_request_v1"
    version = 1

    class error(enum.IntEnum):
        wrong_device = 0
        duplicate_connector = 1
        empty_lease = 2


class WpDrmLeaseRequestV1Proxy(Proxy):
    interface = WpDrmLeaseRequestV1

    @WpDrmLeaseRequestV1.request(
        Argument(ArgumentType.Object, interface=WpDrmLeaseConnectorV1),
    )
    def request_connector(self, connector):
        """Request a connector for this lease

        Indicates that the client would like to lease the given connector. This
        is only used as a suggestion, the compositor may choose to include any
        resources in the lease it issues, or change the set of leased resources
        at any time. Compositors are however encouraged to include the
        requested connector and other resources necessary to drive the
        connected output in the lease.

        Requesting a connector that was created from a different lease device
        than this lease request raises the wrong_device error. Requesting a
        connector twice will raise the duplicate_connector error.

        :param connector:
        :type connector:
            :class:`~pywayland.protocol.drm_lease_v1.WpDrmLeaseConnectorV1`
        """
        self._marshal(0, connector)

    @WpDrmLeaseRequestV1.request(
        Argument(ArgumentType.NewId, interface=WpDrmLeaseV1),
    )
    def submit(self):
        """Submit the lease request

        Submits the lease request and creates a new
        :class:`~pywayland.protocol.drm_lease_v1.WpDrmLeaseV1` object. After
        calling submit the compositor will immediately destroy this object,
        issuing any more requests will cause a wl_diplay error. The compositor
        doesn't make any guarantees about the events of the lease object,
        clients cannot expect an immediate response. Not requesting any
        connectors before submitting the lease request will raise the
        empty_lease error.

        :returns:
            :class:`~pywayland.protocol.drm_lease_v1.WpDrmLeaseV1`
        """
        id = self._marshal_constructor(1, WpDrmLeaseV1)
        return id


class WpDrmLeaseRequestV1Resource(Resource):
    interface = WpDrmLeaseRequestV1


class WpDrmLeaseRequestV1Global(Global):
    interface = WpDrmLeaseRequestV1


WpDrmLeaseRequestV1._gen_c()
WpDrmLeaseRequestV1.proxy_class = WpDrmLeaseRequestV1Proxy
WpDrmLeaseRequestV1.resource_class = WpDrmLeaseRequestV1Resource
WpDrmLeaseRequestV1.global_class = WpDrmLeaseRequestV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2014      Jonas Ådahl
# Copyright © 2015      Red Hat Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource


class ZwpRelativePointerV1(Interface):
    """Relative pointer object

    A wp_relative_pointer object is an extension to the
    :class:`~pywayland.protocol.wayland.WlPointer` interface used for emitting
    relative pointer events. It shares the same focus as
    :class:`~pywayland.protocol.wayland.WlPointer` objects of the same seat and
    will only emit events when it has focus.
    """

    name = "zwp_relative_pointer_v1"
    version = 1


class ZwpRelativePointerV1Proxy(Proxy):
    interface = ZwpRelativePointerV1

    @ZwpRelativePointerV1.request()
    def destroy(self):
        """Release the relative pointer object
        """
        self._marshal(0)
        self._destroy()


class ZwpRelativePointerV1Resource(Resource):
    interface = ZwpRelativePointerV1

    @ZwpRelativePointerV1.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Fixed),
        Argument(ArgumentType.Fixed),
        Argument(ArgumentType.Fixed),
        Argument(ArgumentType.Fixed),
    )
    def relative_motion(self, utime_hi, utime_lo, dx, dy, dx_unaccel, dy_unaccel):
        """Relative pointer motion

        Relative x/y pointer motion from the pointer of the seat associated
        with this object.

        A relative motion is in the same dimension as regular
        :class:`~pywayland.protocol.wayland.WlPointer` motion events, except
        they do not represent an absolute position. For example, moving a
        pointer from (x, y) to (x', y') would have the equivalent relative
        motion (x' - x, y' - y). If a pointer motion caused the absolute
        pointer position to be clipped by for example the edge of the monitor,
        the relative motion is unaffected by the clipping and will represent
        the unclipped motion.

        This event also contains non-accelerated motion deltas. The non-
        accelerated delta is, when applicable, the regular pointer motion delta
        as it was before having applied motion acceleration and other
        transformations such as normalization.

        Note that the non-accelerated delta does not represent 'raw' events as
        they were read from some device. Pointer motion acceleration is device-
        and configuration-specific and non-accelerated deltas and accelerated
        deltas may have the same value on some devices.

        Relative motions are not coupled to :func:`WlPointer.motion()
        <pywayland.protocol.wayland.WlPointer.motion>` events, and can be sent
        in combination with such events, but also independently. There may also
        be scenarios where :func:`WlPointer.motion()
        <pywayland.protocol.wayland.WlPointer.motion>` is sent, but there is no
        relative motion. The order of an absolute and relative motion event
        originating from the same physical motion is not guaranteed.

        If the client needs button events or focus state, it can receive them
        from a :class:`~pywayland.protocol.wayland.WlPointer` object of the
        same seat that the wp_relative_pointer object is associated with.

        :param utime_hi:
            high 32 bits of a 64 bit timestamp with microsecond granularity
        :type utime_hi:
            `ArgumentType.Uint`
        :param utime_lo:
            low 32 bits of a 64 bit timestamp with microsecond granularity
        :type utime_lo:
            `ArgumentType.Uint`
        :param dx:
            the x component of the motion vector
        :type dx:
            `ArgumentType.Fixed`
        :param dy:
            the y component of the motion vector
        :type dy:
            `ArgumentType.Fixed`
        :param dx_unaccel:
            the x component of the unaccelerated motion vector
        :type dx_unaccel:
            `ArgumentType.Fixed`
        :param dy_unaccel:
            the y component of the unaccelerated motion vector
        :type dy_unaccel:
            `ArgumentType.Fixed`
        """
        self._post_event(0, utime_hi, utime_lo, dx, dy, dx_unaccel, dy_unaccel)


class ZwpRelativePointerV1Global(Global):
    interface = ZwpRelativePointerV1


ZwpRelativePointerV1._gen_c()
ZwpRelativePointerV1.proxy_class = ZwpRelativePointerV1Proxy
ZwpRelativePointerV1.resource_class = ZwpRelativePointerV1Resource
ZwpRelativePointerV1.global_class = ZwpRelativePointerV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2017 Red Hat Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.protocol_core import Global, Interface, Proxy, Resource


class ZwpXwaylandKeyboardGrabV1(Interface):
    """Interface for grabbing the keyboard

    A global interface used for grabbing the keyboard.
    """

    name = "zwp_xwayland_keyboard_grab_v1"
    version = 1


class ZwpXwaylandKeyboardGrabV1Proxy(Proxy):
    interface = ZwpXwaylandKeyboardGrabV1

    @ZwpXwaylandKeyboardGrabV1.request()
    def destroy(self):
        """Destroy the grabbed keyboard object

        Destroy the grabbed keyboard object. If applicable, the compositor will
        ungrab the keyboard.
        """
        self._marshal(0)
        self._destroy()


class ZwpXwaylandKeyboardGrabV1Resource(Resource):
    interface = ZwpXwaylandKeyboardGrabV1


class ZwpXwaylandKeyboardGrabV1Global(Global):
    interface = ZwpXwaylandKeyboardGrabV1


ZwpXwaylandKeyboardGrabV1._gen_c()
ZwpXwaylandKeyboardGrabV1.proxy_class = ZwpXwaylandKeyboardGrabV1Proxy
ZwpXwaylandKeyboardGrabV1.resource_class = ZwpXwaylandKeyboardGrabV1Resource
ZwpXwaylandKeyboardGrabV1.global_class = ZwpXwaylandKeyboardGrabV1Global



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pywayland.scanner import Protocol

import os
import pytest
import tempfile

this_dir = os.path.split(__file__)[0]
scanner_dir = os.path.join(this_dir, 'scanner_files')
input_file = os.path.join(scanner_dir, 'test_scanner_input.xml')

interface_tests = [
    "__init__.py",
    "wl_core.py",
    "wl_events.py",
    "wl_requests.py",
    "wl_destructor.py",
    pytest.param("wl_xfail.py", marks=pytest.mark.xfail),
]


@pytest.fixture(scope="session")
def protocol_directory():
    scanner = Protocol.parse_file(input_file)

    imports = {
        interface.name: scanner.name
        for interface in scanner.interface
    }

    generated_files = [
        iface if isinstance(iface, str) else iface.values[0]
        for iface in interface_tests
    ]

    with tempfile.TemporaryDirectory() as output_dir:
        scanner.output(output_dir, imports)

        test_dir = os.path.join(output_dir, "scanner_test")
        assert os.path.exists(test_dir)
        assert set(os.listdir(test_dir)) == set(generated_files)

        yield test_dir


@pytest.mark.parametrize("iface_name", interface_tests)
def test_interface(protocol_directory, iface_name):
    # Get the generated file output
    generated_file = os.path.join(protocol_directory, iface_name)
    with open(generated_file) as f:
        gen_lines = [line.strip('\n') for line in f.readlines()]

    # Get output to check against
    check_file = os.path.join(scanner_dir, iface_name)
    with open(check_file) as f:
        check_lines = [line.strip('\n') for line in f.readlines()]

    # Run through both files, checking each line
    for gen_line, check_line in zip(gen_lines, check_lines):
        assert gen_line == check_line

    # Should both be the same length
    assert len(gen_lines) == len(check_lines)



# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2011 Kristian Høgsberg
# Copyright © 2010-2011 Intel Corporation
# Copyright © 2012-2013 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from .wl_data_device import WlDataDevice
from .wl_data_source import WlDataSource
from .wl_seat import WlSeat


class WlDataDeviceManager(Interface):
    """Data transfer interface

    The :class:`WlDataDeviceManager` is a singleton global object that provides
    access to inter-client data transfer mechanisms such as copy-and-paste and
    drag-and-drop.  These mechanisms are tied to a
    :class:`~pywayland.protocol.wayland.WlSeat` and this interface lets a
    client get a :class:`~pywayland.protocol.wayland.WlDataDevice`
    corresponding to a :class:`~pywayland.protocol.wayland.WlSeat`.

    Depending on the version bound, the objects created from the bound
    :class:`WlDataDeviceManager` object will have different requirements for
    functioning properly. See :func:`WlDataSource.set_actions()
    <pywayland.protocol.wayland.WlDataSource.set_actions>`,
    :func:`WlDataOffer.accept()
    <pywayland.protocol.wayland.WlDataOffer.accept>` and
    :func:`WlDataOffer.finish()
    <pywayland.protocol.wayland.WlDataOffer.finish>` for details.
    """

    name = "wl_data_device_manager"
    version = 3

    class dnd_action(enum.IntFlag):
        none = 0
        copy = 1
        move = 2
        ask = 4


class WlDataDeviceManagerProxy(Proxy):
    interface = WlDataDeviceManager

    @WlDataDeviceManager.request(
        Argument(ArgumentType.NewId, interface=WlDataSource),
    )
    def create_data_source(self):
        """Create a new data source

        Create a new data source.

        :returns:
            :class:`~pywayland.protocol.wayland.WlDataSource` -- data source to
            create
        """
        id = self._marshal_constructor(0, WlDataSource)
        return id

    @WlDataDeviceManager.request(
        Argument(ArgumentType.NewId, interface=WlDataDevice),
        Argument(ArgumentType.Object, interface=WlSeat),
    )
    def get_data_device(self, seat):
        """Create a new data device

        Create a new data device for a given seat.

        :param seat:
            seat associated with the data device
        :type seat:
            :class:`~pywayland.protocol.wayland.WlSeat`
        :returns:
            :class:`~pywayland.protocol.wayland.WlDataDevice` -- data device to
            create
        """
        id = self._marshal_constructor(1, WlDataDevice, seat)
        return id


class WlDataDeviceManagerResource(Resource):
    interface = WlDataDeviceManager


class WlDataDeviceManagerGlobal(Global):
    interface = WlDataDeviceManager


WlDataDeviceManager._gen_c()
WlDataDeviceManager.proxy_class = WlDataDeviceManagerProxy
WlDataDeviceManager.resource_class = WlDataDeviceManagerResource
WlDataDeviceManager.global_class = WlDataDeviceManagerGlobal



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from dataclasses import dataclass
from typing import List, Optional
import xml.etree.ElementTree as ET

from .description import Description
from .entry import Entry
from .element import Element
from .printer import Printer


@dataclass(frozen=True)
class Enum(Element):
    """Scanner for enum objects

    Required attributes: `name` and `since`

    Child elements: `description` and `entry`
    """

    name: str
    since: Optional[str]
    is_bitfield: bool
    description: Optional[Description]
    entry: List[Entry]

    @classmethod
    def parse(cls, element: ET.Element) -> "Enum":
        is_bitfield = cls.parse_optional_attribute(element, "bitfield") == "true"
        return Enum(
            name=cls.parse_attribute(element, "name"),
            since=cls.parse_optional_attribute(element, "since"),
            is_bitfield=is_bitfield,
            description=cls.parse_optional_child(element, Description, "description"),
            entry=cls.parse_repeated_child(element, Entry, "entry"),
        )

    def output(self, printer: Printer) -> None:
        """Generate the output for the enum to the printer"""
        name = self.name if self.name != "version" else "version_"
        if self.is_bitfield:
            printer("class {0}(enum.IntFlag):".format(name))
        else:
            printer("class {0}(enum.IntEnum):".format(name))
        with printer.indented():
            for entry in self.entry:
                entry.output(self.name, printer)



# This file has been autogenerated by the pywayland scanner

# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from .wl_core import WlCore
from .wl_requests import WlRequests


class WlEvents(Interface):
    """Events object

    The interface object with the different types of events.
    """

    name = "wl_events"
    version = 2


class WlEventsProxy(Proxy):
    interface = WlEvents


class WlEventsResource(Resource):
    interface = WlEvents

    @WlEvents.event(
        Argument(ArgumentType.NewId, interface=WlRequests),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.FileDescriptor),
    )
    def send_event(self, id, the_int, the_uint, the_fd):
        """Send the data

        Request for data from the client.  Send the data as the specified mime
        type over the passed file descriptor, then close it.

        :param id:
        :type id:
            :class:`~pywayland.protocol.scanner_test.WlRequests`
        :param the_int:
        :type the_int:
            `ArgumentType.Int`
        :param the_uint:
            the arg summary
        :type the_uint:
            `ArgumentType.Uint`
        :param the_fd:
        :type the_fd:
            `ArgumentType.FileDescriptor`
        """
        self._post_event(0, id, the_int, the_uint, the_fd)

    @WlEvents.event()
    def no_args(self):
        """Event with no args

        An event method that does not have any arguments.
        """
        self._post_event(1)

    @WlEvents.event(
        Argument(ArgumentType.NewId, interface=WlCore),
    )
    def create_id(self, id):
        """Create an id

        With a description

        :param id:
        :type id:
            :class:`~pywayland.protocol.scanner_test.WlCore`
        """
        self._post_event(2, id)

    @WlEvents.event(
        Argument(ArgumentType.NewId, interface=WlCore),
    )
    def create_id2(self, id):
        """Create an id without a description

        :param id:
        :type id:
            :class:`~pywayland.protocol.scanner_test.WlCore`
        """
        self._post_event(3, id)

    @WlEvents.event(
        Argument(ArgumentType.String, nullable=True),
    )
    def allow_null_event(self, null_string):
        """A event with an allowed null argument

        An event where one of the arguments is allowed to be null.

        :param null_string:
        :type null_string:
            `ArgumentType.String` or `None`
        """
        self._post_event(4, null_string)

    @WlEvents.event(
        Argument(ArgumentType.NewId, interface=WlRequests),
        Argument(ArgumentType.Object, interface=WlCore, nullable=True),
    )
    def make_import(self, id, object):
        """Event that causes an import

        An event method that causes an imoprt of other interfaces

        :param id:
        :type id:
            :class:`~pywayland.protocol.scanner_test.WlRequests`
        :param object:
        :type object:
            :class:`~pywayland.protocol.scanner_test.WlCore` or `None`
        """
        self._post_event(5, id, object)

    @WlEvents.event(version=2)
    def versioned(self):
        """A versioned event

        An event that is versioned.
        """
        self._post_event(6)


class WlEventsGlobal(Global):
    interface = WlEvents


WlEvents._gen_c()
WlEvents.proxy_class = WlEventsProxy
WlEvents.resource_class = WlEventsResource
WlEvents.global_class = WlEventsGlobal



# This file has been autogenerated by the pywayland scanner

# Copyright 2016 The Chromium Authors.
# Copyright 2017 Intel Corporation
# Copyright 2018 Collabora, Ltd
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from .zwp_linux_buffer_release_v1 import ZwpLinuxBufferReleaseV1


class ZwpLinuxSurfaceSynchronizationV1(Interface):
    """Per-surface explicit synchronization support

    This object implements per-surface explicit synchronization.

    Synchronization refers to co-ordination of pipelined operations performed
    on buffers. Most GPU clients will schedule an asynchronous operation to
    render to the buffer, then immediately send the buffer to the compositor to
    be attached to a surface.

    In implicit synchronization, ensuring that the rendering operation is
    complete before the compositor displays the buffer is an implementation
    detail handled by either the kernel or userspace graphics driver.

    By contrast, in explicit synchronization, dma_fence objects mark when the
    asynchronous operations are complete. When submitting a buffer, the client
    provides an acquire fence which will be waited on before the compositor
    accesses the buffer. The Wayland server, through a
    :class:`~pywayland.protocol.zwp_linux_explicit_synchronization_unstable_v1.ZwpLinuxBufferReleaseV1`
    object, will inform the client with an event which may be accompanied by a
    release fence, when the compositor will no longer access the buffer
    contents due to the specific commit that requested the release event.

    Each surface can be associated with only one object of this interface at
    any time.

    In version 1 of this interface, explicit synchronization is only guaranteed
    to be supported for buffers created with any version of the wp_linux_dmabuf
    buffer factory. Version 2 additionally guarantees explicit synchronization
    support for opaque EGL buffers, which is a type of platform specific
    buffers described in the EGL_WL_bind_wayland_display extension. Compositors
    are free to support explicit synchronization for additional buffer types.
    """

    name = "zwp_linux_surface_synchronization_v1"
    version = 2

    class error(enum.IntEnum):
        invalid_fence = 0
        duplicate_fence = 1
        duplicate_release = 2
        no_surface = 3
        unsupported_buffer = 4
        no_buffer = 5


class ZwpLinuxSurfaceSynchronizationV1Proxy(Proxy):
    interface = ZwpLinuxSurfaceSynchronizationV1

    @ZwpLinuxSurfaceSynchronizationV1.request()
    def destroy(self):
        """Destroy synchronization object

        Destroy this explicit synchronization object.

        Any fence set by this object with set_acquire_fence since the last
        commit will be discarded by the server. Any fences set by this object
        before the last commit are not affected.

        :class:`~pywayland.protocol.zwp_linux_explicit_synchronization_unstable_v1.ZwpLinuxBufferReleaseV1`
        objects created by this object are not affected by this request.
        """
        self._marshal(0)
        self._destroy()

    @ZwpLinuxSurfaceSynchronizationV1.request(
        Argument(ArgumentType.FileDescriptor),
    )
    def set_acquire_fence(self, fd):
        """Set the acquire fence

        Set the acquire fence that must be signaled before the compositor may
        sample from the buffer attached with :func:`WlSurface.attach()
        <pywayland.protocol.wayland.WlSurface.attach>`. The fence is a
        dma_fence kernel object.

        The acquire fence is double-buffered state, and will be applied on the
        next :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>` request for the
        associated surface. Thus, it applies only to the buffer that is
        attached to the surface at commit time.

        If the provided fd is not a valid dma_fence fd, then an INVALID_FENCE
        error is raised.

        If a fence has already been attached during the same commit cycle, a
        DUPLICATE_FENCE error is raised.

        If the associated :class:`~pywayland.protocol.wayland.WlSurface` was
        destroyed, a NO_SURFACE error is raised.

        If at surface commit time the attached buffer does not support explicit
        synchronization, an UNSUPPORTED_BUFFER error is raised.

        If at surface commit time there is no buffer attached, a NO_BUFFER
        error is raised.

        :param fd:
            acquire fence fd
        :type fd:
            `ArgumentType.FileDescriptor`
        """
        self._marshal(1, fd)

    @ZwpLinuxSurfaceSynchronizationV1.request(
        Argument(ArgumentType.NewId, interface=ZwpLinuxBufferReleaseV1),
    )
    def get_release(self):
        """Release fence for last-attached buffer

        Create a listener for the release of the buffer attached by the client
        with :func:`WlSurface.attach()
        <pywayland.protocol.wayland.WlSurface.attach>`. See
        :class:`~pywayland.protocol.zwp_linux_explicit_synchronization_unstable_v1.ZwpLinuxBufferReleaseV1`
        documentation for more information.

        The release object is double-buffered state, and will be associated
        with the buffer that is attached to the surface at
        :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>` time.

        If a
        :class:`~pywayland.protocol.zwp_linux_explicit_synchronization_unstable_v1.ZwpLinuxBufferReleaseV1`
        object has already been requested for the surface in the same commit
        cycle, a DUPLICATE_RELEASE error is raised.

        If the associated :class:`~pywayland.protocol.wayland.WlSurface` was
        destroyed, a NO_SURFACE error is raised.

        If at surface commit time there is no buffer attached, a NO_BUFFER
        error is raised.

        :returns:
            :class:`~pywayland.protocol.zwp_linux_explicit_synchronization_unstable_v1.ZwpLinuxBufferReleaseV1`
            -- new
            :class:`~pywayland.protocol.zwp_linux_explicit_synchronization_unstable_v1.ZwpLinuxBufferReleaseV1`
            object
        """
        release = self._marshal_constructor(2, ZwpLinuxBufferReleaseV1)
        return release


class ZwpLinuxSurfaceSynchronizationV1Resource(Resource):
    interface = ZwpLinuxSurfaceSynchronizationV1


class ZwpLinuxSurfaceSynchronizationV1Global(Global):
    interface = ZwpLinuxSurfaceSynchronizationV1


ZwpLinuxSurfaceSynchronizationV1._gen_c()
ZwpLinuxSurfaceSynchronizationV1.proxy_class = ZwpLinuxSurfaceSynchronizationV1Proxy
ZwpLinuxSurfaceSynchronizationV1.resource_class = ZwpLinuxSurfaceSynchronizationV1Resource
ZwpLinuxSurfaceSynchronizationV1.global_class = ZwpLinuxSurfaceSynchronizationV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2011 Kristian Høgsberg
# Copyright © 2010-2011 Intel Corporation
# Copyright © 2012-2013 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource


class WlOutput(Interface):
    """Compositor output region

    An output describes part of the compositor geometry.  The compositor works
    in the 'compositor coordinate system' and an output corresponds to a
    rectangular area in that space that is actually visible.  This typically
    corresponds to a monitor that displays part of the compositor space.  This
    object is published as global during start up, or when a monitor is
    hotplugged.
    """

    name = "wl_output"
    version = 3

    class subpixel(enum.IntEnum):
        unknown = 0
        none = 1
        horizontal_rgb = 2
        horizontal_bgr = 3
        vertical_rgb = 4
        vertical_bgr = 5

    class transform(enum.IntEnum):
        normal = 0
        transform_90 = 1
        transform_180 = 2
        transform_270 = 3
        flipped = 4
        flipped_90 = 5
        flipped_180 = 6
        flipped_270 = 7

    class mode(enum.IntFlag):
        current = 0x1
        preferred = 0x2


class WlOutputProxy(Proxy):
    interface = WlOutput

    @WlOutput.request(version=3)
    def release(self):
        """Release the output object

        Using this request a client can tell the server that it is not going to
        use the output object anymore.
        """
        self._marshal(0)
        self._destroy()


class WlOutputResource(Resource):
    interface = WlOutput

    @WlOutput.event(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.String),
        Argument(ArgumentType.String),
        Argument(ArgumentType.Int),
    )
    def geometry(self, x, y, physical_width, physical_height, subpixel, make, model, transform):
        """Properties of the output

        The geometry event describes geometric properties of the output. The
        event is sent when binding to the output object and whenever any of the
        properties change.

        The physical size can be set to zero if it doesn't make sense for this
        output (e.g. for projectors or virtual outputs).

        Note: :class:`WlOutput` only advertises partial information about the
        output position and identification. Some compositors, for instance
        those not implementing a desktop-style output layout or those exposing
        virtual outputs, might fake this information. Instead of using x and y,
        clients should use xdg_output.logical_position. Instead of using make
        and model, clients should use xdg_output.name and
        xdg_output.description.

        :param x:
            x position within the global compositor space
        :type x:
            `ArgumentType.Int`
        :param y:
            y position within the global compositor space
        :type y:
            `ArgumentType.Int`
        :param physical_width:
            width in millimeters of the output
        :type physical_width:
            `ArgumentType.Int`
        :param physical_height:
            height in millimeters of the output
        :type physical_height:
            `ArgumentType.Int`
        :param subpixel:
            subpixel orientation of the output
        :type subpixel:
            `ArgumentType.Int`
        :param make:
            textual description of the manufacturer
        :type make:
            `ArgumentType.String`
        :param model:
            textual description of the model
        :type model:
            `ArgumentType.String`
        :param transform:
            transform that maps framebuffer to output
        :type transform:
            `ArgumentType.Int`
        """
        self._post_event(0, x, y, physical_width, physical_height, subpixel, make, model, transform)

    @WlOutput.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def mode(self, flags, width, height, refresh):
        """Advertise available modes for the output

        The mode event describes an available mode for the output.

        The event is sent when binding to the output object and there will
        always be one mode, the current mode.  The event is sent again if an
        output changes mode, for the mode that is now current.  In other words,
        the current mode is always the last mode that was received with the
        current flag set.

        Non-current modes are deprecated. A compositor can decide to only
        advertise the current mode and never send other modes. Clients should
        not rely on non-current modes.

        The size of a mode is given in physical hardware units of the output
        device. This is not necessarily the same as the output size in the
        global compositor space. For instance, the output may be scaled, as
        described in :func:`WlOutput.scale()`, or transformed, as described in
        :func:`WlOutput.transform()`. Clients willing to retrieve the output
        size in the global compositor space should use xdg_output.logical_size
        instead.

        The vertical refresh rate can be set to zero if it doesn't make sense
        for this output (e.g. for virtual outputs).

        Clients should not use the refresh rate to schedule frames. Instead,
        they should use the :func:`WlSurface.frame()
        <pywayland.protocol.wayland.WlSurface.frame>` event or the
        presentation-time protocol.

        Note: this information is not always meaningful for all outputs. Some
        compositors, such as those exposing virtual outputs, might fake the
        refresh rate or the size.

        :param flags:
            bitfield of mode flags
        :type flags:
            `ArgumentType.Uint`
        :param width:
            width of the mode in hardware units
        :type width:
            `ArgumentType.Int`
        :param height:
            height of the mode in hardware units
        :type height:
            `ArgumentType.Int`
        :param refresh:
            vertical refresh rate in mHz
        :type refresh:
            `ArgumentType.Int`
        """
        self._post_event(1, flags, width, height, refresh)

    @WlOutput.event(version=2)
    def done(self):
        """Sent all information about output

        This event is sent after all other properties have been sent after
        binding to the output object and after any other property changes done
        after that. This allows changes to the output properties to be seen as
        atomic, even if they happen via multiple events.
        """
        self._post_event(2)

    @WlOutput.event(
        Argument(ArgumentType.Int),
        version=2,
    )
    def scale(self, factor):
        """Output scaling properties

        This event contains scaling geometry information that is not in the
        geometry event. It may be sent after binding the output object or if
        the output scale changes later. If it is not sent, the client should
        assume a scale of 1.

        A scale larger than 1 means that the compositor will automatically
        scale surface buffers by this amount when rendering. This is used for
        very high resolution displays where applications rendering at the
        native resolution would be too small to be legible.

        It is intended that scaling aware clients track the current output of a
        surface, and if it is on a scaled output it should use
        :func:`WlSurface.set_buffer_scale()
        <pywayland.protocol.wayland.WlSurface.set_buffer_scale>` with the scale
        of the output. That way the compositor can avoid scaling the surface,
        and the client can supply a higher detail image.

        :param factor:
            scaling factor of output
        :type factor:
            `ArgumentType.Int`
        """
        self._post_event(3, factor)


class WlOutputGlobal(Global):
    interface = WlOutput


WlOutput._gen_c()
WlOutput.proxy_class = WlOutputProxy
WlOutput.resource_class = WlOutputResource
WlOutput.global_class = WlOutputGlobal



# This file has been autogenerated by the pywayland scanner

# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from .wl_core import WlCore
from .wl_events import WlEvents


class WlRequests(Interface):
    """Request object

    The interface object with the different types of requests.
    """

    name = "wl_requests"
    version = 2


class WlRequestsProxy(Proxy):
    interface = WlRequests

    @WlRequests.request(
        Argument(ArgumentType.NewId, interface=WlCore),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.FileDescriptor),
    )
    def make_request(self, the_int, the_uint, the_fd):
        """A request

        The request asks the server for an event.

        :param the_int:
        :type the_int:
            `ArgumentType.Int`
        :param the_uint:
            the arg summary
        :type the_uint:
            `ArgumentType.Uint`
        :param the_fd:
        :type the_fd:
            `ArgumentType.FileDescriptor`
        :returns:
            :class:`~pywayland.protocol.scanner_test.WlCore`
        """
        id = self._marshal_constructor(0, WlCore, the_int, the_uint, the_fd)
        return id

    @WlRequests.request()
    def no_args(self):
        """Request with no args

        A request method that does not have any arguments.
        """
        self._marshal(1)

    @WlRequests.request(
        Argument(ArgumentType.NewId, interface=WlCore),
    )
    def create_id(self):
        """Create an id

        With a description

        :returns:
            :class:`~pywayland.protocol.scanner_test.WlCore`
        """
        id = self._marshal_constructor(2, WlCore)
        return id

    @WlRequests.request(
        Argument(ArgumentType.NewId, interface=WlCore),
    )
    def create_id2(self):
        """Create an id without a description

        :returns:
            :class:`~pywayland.protocol.scanner_test.WlCore`
        """
        id = self._marshal_constructor(3, WlCore)
        return id

    @WlRequests.request(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.String, nullable=True),
    )
    def allow_null(self, serial, mime_type):
        """Request that allows for null arguments

        A request where one of the arguments is allowed to be null.

        :param serial:
        :type serial:
            `ArgumentType.Uint`
        :param mime_type:
        :type mime_type:
            `ArgumentType.String` or `None`
        """
        self._marshal(4, serial, mime_type)

    @WlRequests.request(
        Argument(ArgumentType.NewId, interface=WlEvents),
        Argument(ArgumentType.Object, interface=WlCore, nullable=True),
    )
    def make_import(self, object):
        """Request that causes an import

        A request method that causes an imoprt of other interfaces, both as a
        new_id and as an object.

        :param object:
        :type object:
            :class:`~pywayland.protocol.scanner_test.WlCore` or `None`
        :returns:
            :class:`~pywayland.protocol.scanner_test.WlEvents`
        """
        id = self._marshal_constructor(5, WlEvents, object)
        return id

    @WlRequests.request(version=2)
    def versioned(self):
        """A versioned request

        A request that is versioned.
        """
        self._marshal(6)

    @WlRequests.request(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.NewId),
    )
    def new_id_no_interface(self, name, interface, version):
        """Create a new id, but with no interface

        A method with an argument for a new_id, but with no corresponding
        interface (c.f. wl_registry.bind).

        :param name:
        :type name:
            `ArgumentType.Uint`
        :param interface:
            Interface name
        :type interface:
            `string`
        :param version:
            Interface version
        :type version:
            `int`
        :returns:
            :class:`pywayland.client.proxy.Proxy` of specified Interface
        """
        id = self._marshal_constructor(7, interface, name, interface.name, version)
        return id


class WlRequestsResource(Resource):
    interface = WlRequests


class WlRequestsGlobal(Global):
    interface = WlRequests


WlRequests._gen_c()
WlRequests.proxy_class = WlRequestsProxy
WlRequests.resource_class = WlRequestsResource
WlRequests.global_class = WlRequestsGlobal



# Copyright 2019 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""The core objects used by the protocol definitions

This defines the core objects that are used in building up the protocols and
interfaces.  Each interface has a set of requests and events that are
associated with it.  These functions are invoked from the corresponding proxy
and resource instances.
"""

from .argument import Argument, ArgumentType  # noqa: F401
from .globals import Global  # noqa: F401
from .interface import Interface  # noqa: F401
from .proxy import Proxy  # noqa: F401
from .resource import Resource  # noqa: F401



# This file has been autogenerated by the pywayland scanner

# Copyright © 2018 NXP
# Copyright © 2019 Status Research & Development GmbH.
# Copyright © 2021 Xaver Hugl
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from .wp_drm_lease_connector_v1 import WpDrmLeaseConnectorV1
from .wp_drm_lease_request_v1 import WpDrmLeaseRequestV1


class WpDrmLeaseDeviceV1(Interface):
    """Lease device

    This protocol is used by Wayland compositors which act as Direct
    Renderering Manager (DRM) masters to lease DRM resources to Wayland
    clients.

    The compositor will advertise one :class:`WpDrmLeaseDeviceV1` global for
    each DRM node. Some time after a client binds to the
    :class:`WpDrmLeaseDeviceV1` global, the compositor will send a drm_fd event
    followed by zero, one or more connector events. After all currently
    available connectors have been sent, the compositor will send a
    :func:`WpDrmLeaseDeviceV1.done()` event.

    When the list of connectors available for lease changes the compositor will
    send :func:`WpDrmLeaseDeviceV1.connector()` events for added connectors and
    :func:`WpDrmLeaseConnectorV1.withdrawn()
    <pywayland.protocol.drm_lease_v1.WpDrmLeaseConnectorV1.withdrawn>` events
    for removed connectors, followed by a :func:`WpDrmLeaseDeviceV1.done()`
    event.

    The compositor will indicate when a device is gone by removing the global
    via a :func:`WlRegistry.global_remove()
    <pywayland.protocol.wayland.WlRegistry.global_remove>` event. Upon
    receiving this event, the client should destroy any matching
    :class:`WpDrmLeaseDeviceV1` object.

    To destroy a :class:`WpDrmLeaseDeviceV1` object, the client must first
    issue a release request. Upon receiving this request, the compositor will
    immediately send a released event and destroy the object. The client must
    continue to process and discard drm_fd and connector events until it
    receives the released event. Upon receiving the released event, the client
    can safely cleanup any client-side resources.

    Warning! The protocol described in this file is currently in the testing
    phase. Backward compatible changes may be added together with the
    corresponding interface version bump. Backward incompatible changes can
    only be done by creating a new major version of the extension.
    """

    name = "wp_drm_lease_device_v1"
    version = 1


class WpDrmLeaseDeviceV1Proxy(Proxy):
    interface = WpDrmLeaseDeviceV1

    @WpDrmLeaseDeviceV1.request(
        Argument(ArgumentType.NewId, interface=WpDrmLeaseRequestV1),
    )
    def create_lease_request(self):
        """Create a lease request object

        Creates a lease request object.

        See the documentation for
        :class:`~pywayland.protocol.drm_lease_v1.WpDrmLeaseRequestV1` for
        details.

        :returns:
            :class:`~pywayland.protocol.drm_lease_v1.WpDrmLeaseRequestV1`
        """
        id = self._marshal_constructor(0, WpDrmLeaseRequestV1)
        return id

    @WpDrmLeaseDeviceV1.request()
    def release(self):
        """Release this object

        Indicates the client no longer wishes to use this object. In response
        the compositor will immediately send the released event and destroy
        this object. It can however not guarantee that the client won't receive
        connector events before the released event. The client must not send
        any requests after this one, doing so will raise a
        :class:`~pywayland.protocol.wayland.WlDisplay` error. Existing
        connectors, lease request and leases will not be affected.
        """
        self._marshal(1)


class WpDrmLeaseDeviceV1Resource(Resource):
    interface = WpDrmLeaseDeviceV1

    @WpDrmLeaseDeviceV1.event(
        Argument(ArgumentType.FileDescriptor),
    )
    def drm_fd(self, fd):
        """Open a non-master fd for this drm node

        The compositor will send this event when the
        :class:`WpDrmLeaseDeviceV1` global is bound, although there are no
        guarantees as to how long this takes - the compositor might need to
        wait until regaining DRM master. The included fd is a non-master DRM
        file descriptor opened for this device and the compositor must not
        authenticate it. The purpose of this event is to give the client the
        ability to query DRM and discover information which may help them pick
        the appropriate DRM device or select the appropriate connectors
        therein.

        :param fd:
            DRM file descriptor
        :type fd:
            `ArgumentType.FileDescriptor`
        """
        self._post_event(0, fd)

    @WpDrmLeaseDeviceV1.event(
        Argument(ArgumentType.NewId, interface=WpDrmLeaseConnectorV1),
    )
    def connector(self, id):
        """Advertise connectors available for leases

        The compositor will use this event to advertise connectors available
        for lease by clients. This object may be passed into a lease request to
        indicate the client would like to lease that connector, see
        :func:`WpDrmLeaseRequestV1.request_connector()
        <pywayland.protocol.drm_lease_v1.WpDrmLeaseRequestV1.request_connector>`
        for details. While the compositor will make a best effort to not send
        disconnected connectors, no guarantees can be made.

        The compositor must send the drm_fd event before sending connectors.
        After the drm_fd event it will send all available connectors but may
        send additional connectors at any time.

        :param id:
        :type id:
            :class:`~pywayland.protocol.drm_lease_v1.WpDrmLeaseConnectorV1`
        """
        self._post_event(1, id)

    @WpDrmLeaseDeviceV1.event()
    def done(self):
        """Signals grouping of connectors

        The compositor will send this event to indicate that it has sent all
        currently available connectors after the client binds to the global or
        when it updates the connector list, for example on hotplug, drm master
        change or when a leased connector becomes available again. It will
        similarly send this event to group
        :func:`WpDrmLeaseConnectorV1.withdrawn()
        <pywayland.protocol.drm_lease_v1.WpDrmLeaseConnectorV1.withdrawn>`
        events of connectors of this device.
        """
        self._post_event(2)

    @WpDrmLeaseDeviceV1.event()
    def released(self):
        """The compositor has finished using the device

        This event is sent in response to the release request and indicates
        that the compositor is done sending connector events. The compositor
        will destroy this object immediately after sending the event and it
        will become invalid. The client should release any resources associated
        with this device after receiving this event.
        """
        self._post_event(3)


class WpDrmLeaseDeviceV1Global(Global):
    interface = WpDrmLeaseDeviceV1


WpDrmLeaseDeviceV1._gen_c()
WpDrmLeaseDeviceV1.proxy_class = WpDrmLeaseDeviceV1Proxy
WpDrmLeaseDeviceV1.resource_class = WpDrmLeaseDeviceV1Resource
WpDrmLeaseDeviceV1.global_class = WpDrmLeaseDeviceV1Global



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import xml.etree.ElementTree as ET
from dataclasses import dataclass
from typing import Optional

from .element import Element
from .description import Description
from .printer import Printer


@dataclass(frozen=True)
class Entry(Element):
    """Scanner for enum entries

    Required attributes: `name` and `value`

    Optional attributes: `summary` and `since`

    Child elements: `description`
    """

    name: str
    value: str
    summary: Optional[str]
    since: Optional[str]
    description: Optional[Description]

    @classmethod
    def parse(cls, element: ET.Element) -> "Entry":
        return Entry(
            name=cls.parse_attribute(element, "name"),
            value=cls.parse_attribute(element, "value"),
            summary=cls.parse_optional_attribute(element, "summary"),
            since=cls.parse_optional_attribute(element, "since"),
            description=cls.parse_optional_child(element, Description, "description"),
        )

    def output(self, enum_name: str, printer: Printer) -> None:
        """Generate the output for the entry in the enum"""
        # keep base 10 ints unchanged, but ensure that hexidecimal ints are
        # formatted 0xABC
        if self.value[:2] == "0x":
            value = "0x{}".format(self.value[2:].upper())
        else:
            value = self.value

        try:
            int(self.name)
            printer("{}_{} = {}".format(enum_name, self.name, value))
        except ValueError:
            if self.name == "name":
                name = "name_"
            else:
                name = self.name
            printer("{} = {}".format(name, value))



# This file has been autogenerated by the pywayland scanner

# Copyright 2014 © Stephen "Lyude" Chandler Paul
# Copyright 2015-2016 © Red Hat, Inc.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource


class ZwpTabletPadRingV2(Interface):
    """Pad ring

    A circular interaction area, such as the touch ring on the Wacom Intuos Pro
    series tablets.

    Events on a ring are logically grouped by the wl_tablet_pad_ring.frame
    event.
    """

    name = "zwp_tablet_pad_ring_v2"
    version = 1

    class source(enum.IntEnum):
        finger = 1


class ZwpTabletPadRingV2Proxy(Proxy):
    interface = ZwpTabletPadRingV2

    @ZwpTabletPadRingV2.request(
        Argument(ArgumentType.String),
        Argument(ArgumentType.Uint),
    )
    def set_feedback(self, description, serial):
        """Set compositor feedback

        Request that the compositor use the provided feedback string associated
        with this ring. This request should be issued immediately after a
        wp_tablet_pad_group.mode_switch event from the corresponding group is
        received, or whenever the ring is mapped to a different action. See
        wp_tablet_pad_group.mode_switch for more details.

        Clients are encouraged to provide context-aware descriptions for the
        actions associated with the ring; compositors may use this information
        to offer visual feedback about the button layout (eg. on-screen
        displays).

        The provided string 'description' is a UTF-8 encoded string to be
        associated with this ring, and is considered user-visible; general
        internationalization rules apply.

        The serial argument will be that of the last
        wp_tablet_pad_group.mode_switch event received for the group of this
        ring. Requests providing other serials than the most recent one will be
        ignored.

        :param description:
            ring description
        :type description:
            `ArgumentType.String`
        :param serial:
            serial of the mode switch event
        :type serial:
            `ArgumentType.Uint`
        """
        self._marshal(0, description, serial)

    @ZwpTabletPadRingV2.request()
    def destroy(self):
        """Destroy the ring object

        This destroys the client's resource for this ring object.
        """
        self._marshal(1)
        self._destroy()


class ZwpTabletPadRingV2Resource(Resource):
    interface = ZwpTabletPadRingV2

    @ZwpTabletPadRingV2.event(
        Argument(ArgumentType.Uint),
    )
    def source(self, source):
        """Ring event source

        Source information for ring events.

        This event does not occur on its own. It is sent before a
        wp_tablet_pad_ring.frame event and carries the source information for
        all events within that frame.

        The source specifies how this event was generated. If the source is
        wp_tablet_pad_ring.source.finger, a wp_tablet_pad_ring.stop event will
        be sent when the user lifts the finger off the device.

        This event is optional. If the source is unknown for an interaction, no
        event is sent.

        :param source:
            the event source
        :type source:
            `ArgumentType.Uint`
        """
        self._post_event(0, source)

    @ZwpTabletPadRingV2.event(
        Argument(ArgumentType.Fixed),
    )
    def angle(self, degrees):
        """Angle changed

        Sent whenever the angle on a ring changes.

        The angle is provided in degrees clockwise from the logical north of
        the ring in the pad's current rotation.

        :param degrees:
            the current angle in degrees
        :type degrees:
            `ArgumentType.Fixed`
        """
        self._post_event(1, degrees)

    @ZwpTabletPadRingV2.event()
    def stop(self):
        """Interaction stopped

        Stop notification for ring events.

        For some wp_tablet_pad_ring.source types, a wp_tablet_pad_ring.stop
        event is sent to notify a client that the interaction with the ring has
        terminated. This enables the client to implement kinetic scrolling. See
        the wp_tablet_pad_ring.source documentation for information on when
        this event may be generated.

        Any wp_tablet_pad_ring.angle events with the same source after this
        event should be considered as the start of a new interaction.
        """
        self._post_event(2)

    @ZwpTabletPadRingV2.event(
        Argument(ArgumentType.Uint),
    )
    def frame(self, time):
        """End of a ring event sequence

        Indicates the end of a set of ring events that logically belong
        together. A client is expected to accumulate the data in all events
        within the frame before proceeding.

        All wp_tablet_pad_ring events before a wp_tablet_pad_ring.frame event
        belong logically together. For example, on termination of a finger
        interaction on a ring the compositor will send a
        wp_tablet_pad_ring.source event, a wp_tablet_pad_ring.stop event and a
        wp_tablet_pad_ring.frame event.

        A wp_tablet_pad_ring.frame event is sent for every logical event group,
        even if the group only contains a single wp_tablet_pad_ring event.
        Specifically, a client may get a sequence: angle, frame, angle, frame,
        etc.

        :param time:
            timestamp with millisecond granularity
        :type time:
            `ArgumentType.Uint`
        """
        self._post_event(3, time)


class ZwpTabletPadRingV2Global(Global):
    interface = ZwpTabletPadRingV2


ZwpTabletPadRingV2._gen_c()
ZwpTabletPadRingV2.proxy_class = ZwpTabletPadRingV2Proxy
ZwpTabletPadRingV2.resource_class = ZwpTabletPadRingV2Resource
ZwpTabletPadRingV2.global_class = ZwpTabletPadRingV2Global
